# Copyright 2013 OpenStack Foundation
# Copyright 2013 IBM Corp
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import io
import random

from oslo_log import log as logging
from tempest.api.image import base
from tempest.common import waiters
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc

CONF = config.CONF
LOG = logging.getLogger(__name__)


class ImportImagesTest(base.BaseV2ImageTest):
    """Here we test the import operations for image"""

    @classmethod
    def skip_checks(cls):
        super(ImportImagesTest, cls).skip_checks()
        if not CONF.image_feature_enabled.import_image:
            skip_msg = (
                "%s skipped as image import is not available" % cls.__name__)
            raise cls.skipException(skip_msg)

    @classmethod
    def resource_setup(cls):
        super(ImportImagesTest, cls).resource_setup()
        cls.available_import_methods = cls.client.info_import()[
            'import-methods']['value']
        if not cls.available_import_methods:
            raise cls.skipException('Server does not support '
                                    'any import method')

    def _create_image(self):
        # Create image
        uuid = '00000000-1111-2222-3333-444455556666'
        image_name = data_utils.rand_name('image')
        container_format = CONF.image.container_formats[0]
        disk_format = CONF.image.disk_formats[0]
        image = self.create_image(name=image_name,
                                  container_format=container_format,
                                  disk_format=disk_format,
                                  visibility='private',
                                  ramdisk_id=uuid)
        self.assertIn('name', image)
        self.assertEqual(image_name, image['name'])
        self.assertIn('visibility', image)
        self.assertEqual('private', image['visibility'])
        self.assertIn('status', image)
        self.assertEqual('queued', image['status'])
        return image

    def _require_import_method(self, method):
        if method not in self.available_import_methods:
            raise self.skipException('Server does not support '
                                     '%s import method' % method)

    def _stage_and_check(self):
        image = self._create_image()
        # Stage image data
        file_content = data_utils.random_bytes()
        image_file = io.BytesIO(file_content)
        self.client.stage_image_file(image['id'], image_file)
        # Check image status is 'uploading'
        body = self.client.show_image(image['id'])
        self.assertEqual(image['id'], body['id'])
        self.assertEqual('uploading', body['status'])
        return image['id']

    @decorators.idempotent_id('32ca0c20-e16f-44ac-8590-07869c9b4cc2')
    def test_image_glance_direct_import(self):
        """Test 'glance-direct' import functionalities

        Create image, stage image data, import image and verify
        that import succeeded.
        """
        self._require_import_method('glance-direct')

        image_id = self._stage_and_check()
        # import image from staging to backend
        resp = self.client.image_import(image_id, method='glance-direct')
        waiters.wait_for_image_imported_to_stores(self.client, image_id)

        if not self.versions_client.has_version('2.12'):
            # API is not new enough to support image/tasks API
            LOG.info('Glance does not support v2.12, so I am unable to '
                     'validate the image/tasks API.')
            return

        # Make sure we can access the task and that some of the key
        # fields look legit.
        tasks = self.client.show_image_tasks(image_id)
        self.assertEqual(1, len(tasks['tasks']))
        task = tasks['tasks'][0]
        self.assertEqual('success', task['status'])
        self.assertEqual(resp.response['x-openstack-request-id'],
                         task['request_id'])
        self.assertEqual('glance-direct',
                         task['input']['import_req']['method']['name'])

    @decorators.idempotent_id('f6feb7a4-b04f-4706-a011-206129f83e62')
    def test_image_web_download_import(self):
        """Test 'web-download' import functionalities

        Create image, import image and verify that import
        succeeded.
        """
        self._require_import_method('web-download')

        image = self._create_image()
        # Now try to get image details
        body = self.client.show_image(image['id'])
        self.assertEqual(image['id'], body['id'])
        self.assertEqual('queued', body['status'])
        # import image from web to backend
        image_uri = CONF.image.http_image
        self.client.image_import(image['id'], method='web-download',
                                 image_uri=image_uri)
        waiters.wait_for_image_imported_to_stores(self.client, image['id'])

    @decorators.idempotent_id('e04761a1-22af-42c2-b8bc-a34a3f12b585')
    def test_remote_import(self):
        """Test image import against a different worker than stage.

        This creates and stages an image against the primary API worker,
        but then calls import on a secondary worker (if available) to
        test that distributed image import works (i.e. proxies the import
        request to the proper worker).
        """
        self._require_import_method('glance-direct')

        if not CONF.image.alternate_image_endpoint:
            raise self.skipException('No image_remote service to test '
                                     'against')

        image_id = self._stage_and_check()
        # import image from staging to backend, but on the alternate worker
        self.os_primary.image_client_remote.image_import(
            image_id, method='glance-direct')
        waiters.wait_for_image_imported_to_stores(self.client, image_id)

    @decorators.idempotent_id('44d60544-1524-42f7-8899-315301105dd8')
    def test_remote_delete(self):
        """Test image delete against a different worker than stage.

        This creates and stages an image against the primary API worker,
        but then calls delete on a secondary worker (if available) to
        test that distributed image import works (i.e. proxies the delete
        request to the proper worker).
        """
        self._require_import_method('glance-direct')

        if not CONF.image.alternate_image_endpoint:
            raise self.skipException('No image_remote service to test '
                                     'against')

        image_id = self._stage_and_check()
        # delete image from staging to backend, but on the alternate worker
        self.os_primary.image_client_remote.delete_image(image_id)
        self.client.wait_for_resource_deletion(image_id)


class MultiStoresImportImagesTest(base.BaseV2ImageTest):
    """Test importing image in multiple stores"""
    @classmethod
    def skip_checks(cls):
        super(MultiStoresImportImagesTest, cls).skip_checks()
        if not CONF.image_feature_enabled.import_image:
            skip_msg = (
                "%s skipped as image import is not available" % cls.__name__)
            raise cls.skipException(skip_msg)

    @classmethod
    def resource_setup(cls):
        super(MultiStoresImportImagesTest, cls).resource_setup()
        cls.available_import_methods = cls.client.info_import()[
            'import-methods']['value']
        if not cls.available_import_methods:
            raise cls.skipException('Server does not support '
                                    'any import method')

        # NOTE(pdeore): Skip if glance-direct import method and mutlistore
        # are not enabled/configured, or only one store is configured in
        # multiple stores setup.
        cls.available_stores = cls.get_available_stores()
        if ('glance-direct' not in cls.available_import_methods or
                not len(cls.available_stores) > 1):
            raise cls.skipException(
                'Either glance-direct import method not present in %s or '
                'None or only one store is '
                'configured %s' % (cls.available_import_methods,
                                   cls.available_stores))

    def _create_and_stage_image(self, all_stores=False):
        """Create Image & stage image file for glance-direct import method."""
        image_name = data_utils.rand_name('test-image')
        container_format = CONF.image.container_formats[0]
        disk_format = CONF.image.disk_formats[0]
        image = self.create_image(name=image_name,
                                  container_format=container_format,
                                  disk_format=disk_format,
                                  visibility='private')
        self.assertEqual('queued', image['status'])

        self.client.stage_image_file(
            image['id'],
            io.BytesIO(data_utils.random_bytes()))
        # Check image status is 'uploading'
        body = self.client.show_image(image['id'])
        self.assertEqual(image['id'], body['id'])
        self.assertEqual('uploading', body['status'])

        if all_stores:
            stores_list = ','.join([store['id']
                                    for store in self.available_stores])
        else:
            stores = [store['id'] for store in self.available_stores]
            stores_list = stores[::len(stores) - 1]

        return body, stores_list

    @decorators.idempotent_id('bf04ff00-3182-47cb-833a-f1c6767b47fd')
    def test_glance_direct_import_image_to_all_stores(self):
        """Test image is imported in all available stores

        Create image, import image to all available stores using glance-direct
        import method and verify that import succeeded.
        """
        image, stores = self._create_and_stage_image(all_stores=True)

        self.client.image_import(
            image['id'], method='glance-direct', all_stores=True)

        waiters.wait_for_image_imported_to_stores(self.client,
                                                  image['id'], stores)

    @decorators.idempotent_id('82fb131a-dd2b-11ea-aec7-340286b6c574')
    def test_glance_direct_import_image_to_specific_stores(self):
        """Test image is imported in all available stores

        Create image, import image to specified store(s) using glance-direct
        import method and verify that import succeeded.
        """
        image, stores = self._create_and_stage_image()
        self.client.image_import(image['id'], method='glance-direct',
                                 stores=stores)

        waiters.wait_for_image_imported_to_stores(self.client, image['id'],
                                                  (','.join(stores)))


class BasicOperationsImagesTest(base.BaseV2ImageTest):
    """Here we test the basic operations of images"""

    @decorators.attr(type='smoke')
    @decorators.idempotent_id('139b765e-7f3d-4b3d-8b37-3ca3876ee318')
    def test_register_upload_get_image_file(self):
        """Here we test these functionalities

        Register image, upload the image file, get image and get image
        file api's
        """

        uuid = '00000000-1111-2222-3333-444455556666'
        image_name = data_utils.rand_name('image')
        container_format = CONF.image.container_formats[0]
        disk_format = CONF.image.disk_formats[0]
        image = self.create_image(name=image_name,
                                  container_format=container_format,
                                  disk_format=disk_format,
                                  visibility='private',
                                  ramdisk_id=uuid)
        self.assertIn('name', image)
        self.assertEqual(image_name, image['name'])
        self.assertIn('visibility', image)
        self.assertEqual('private', image['visibility'])
        self.assertIn('status', image)
        self.assertEqual('queued', image['status'])

        # NOTE: This Glance API returns different status codes for image
        # condition. In this empty data case, Glance should return 204,
        # so here should check the status code.
        image_file = self.client.show_image_file(image['id'])
        self.assertEqual(0, len(image_file.data))
        self.assertEqual(204, image_file.response.status)

        # Now try uploading an image file
        file_content = data_utils.random_bytes()
        image_file = io.BytesIO(file_content)
        self.client.store_image_file(image['id'], image_file)

        # Now try to get image details
        body = self.client.show_image(image['id'])
        self.assertEqual(image['id'], body['id'])
        self.assertEqual(image_name, body['name'])
        self.assertEqual(uuid, body['ramdisk_id'])
        self.assertIn('size', body)
        self.assertEqual(1024, body.get('size'))

        # Now try get image file
        # NOTE: This Glance API returns different status codes for image
        # condition. In this non-empty data case, Glance should return 200,
        # so here should check the status code.
        body = self.client.show_image_file(image['id'])
        self.assertEqual(file_content, body.data)
        self.assertEqual(200, body.response.status)

    @decorators.attr(type='smoke')
    @decorators.idempotent_id('f848bb94-1c6e-45a4-8726-39e3a5b23535')
    def test_delete_image(self):
        """Test deleting an image by image_id"""
        # Create image
        image_name = data_utils.rand_name('image')
        container_format = CONF.image.container_formats[0]
        disk_format = CONF.image.disk_formats[0]
        image = self.create_image(name=image_name,
                                  container_format=container_format,
                                  disk_format=disk_format,
                                  visibility='private')
        # Delete Image
        self.client.delete_image(image['id'])
        self.client.wait_for_resource_deletion(image['id'])

        # Verifying deletion
        images = self.client.list_images()['images']
        images_id = [item['id'] for item in images]
        self.assertNotIn(image['id'], images_id)

    @decorators.attr(type='smoke')
    @decorators.idempotent_id('f66891a7-a35c-41a8-b590-a065c2a1caa6')
    def test_update_image(self):
        """Test updating an image by image_id"""
        # Create image
        image_name = data_utils.rand_name('image')
        container_format = CONF.image.container_formats[0]
        disk_format = CONF.image.disk_formats[0]
        image = self.create_image(name=image_name,
                                  container_format=container_format,
                                  disk_format=disk_format,
                                  visibility='private')
        self.assertEqual('queued', image['status'])

        # Update Image
        new_image_name = data_utils.rand_name('new-image')
        self.client.update_image(image['id'], [
            dict(replace='/name', value=new_image_name)])

        # Verifying updating

        body = self.client.show_image(image['id'])
        self.assertEqual(image['id'], body['id'])
        self.assertEqual(new_image_name, body['name'])

    @decorators.idempotent_id('951ebe01-969f-4ea9-9898-8a3f1f442ab0')
    def test_deactivate_reactivate_image(self):
        """Test deactivating and reactivating an image"""
        # Create image
        image_name = data_utils.rand_name('image')
        image = self.create_image(name=image_name,
                                  container_format='bare',
                                  disk_format='raw',
                                  visibility='private')

        # Upload an image file
        content = data_utils.random_bytes()
        image_file = io.BytesIO(content)
        self.client.store_image_file(image['id'], image_file)

        # Deactivate image
        self.client.deactivate_image(image['id'])
        body = self.client.show_image(image['id'])
        self.assertEqual("deactivated", body['status'])

        # User unable to download deactivated image
        self.assertRaises(lib_exc.Forbidden, self.client.show_image_file,
                          image['id'])

        # Reactivate image
        self.client.reactivate_image(image['id'])
        body = self.client.show_image(image['id'])
        self.assertEqual("active", body['status'])

        # User able to download image after reactivation
        body = self.client.show_image_file(image['id'])
        self.assertEqual(content, body.data)


class ListUserImagesTest(base.BaseV2ImageTest):
    """Here we test the listing of image information"""

    @classmethod
    def resource_setup(cls):
        super(ListUserImagesTest, cls).resource_setup()
        # We add a few images here to test the listing functionality of
        # the images API
        container_fmts = CONF.image.container_formats
        disk_fmts = CONF.image.disk_formats
        all_pairs = [(container_fmt, disk_fmt)
                     for container_fmt in container_fmts
                     for disk_fmt in disk_fmts]

        for (container_fmt, disk_fmt) in all_pairs[:6]:
            LOG.debug("Creating an image "
                      "(Container format: %s, Disk format: %s).",
                      container_fmt, disk_fmt)
            cls._create_standard_image(container_fmt, disk_fmt)

    @classmethod
    def _create_standard_image(cls, container_format, disk_format):
        """Create a new standard image and return the newly-registered image-id

        Note that the size of the new image is a random number between
        1024 and 4096
        """
        size = random.randint(1024, 4096)
        image_file = io.BytesIO(data_utils.random_bytes(size))
        tags = [data_utils.rand_name('tag'), data_utils.rand_name('tag')]
        image = cls.create_image(container_format=container_format,
                                 disk_format=disk_format,
                                 visibility='private',
                                 tags=tags)
        cls.client.store_image_file(image['id'], data=image_file)
        # Keep the data of one test image so it can be used to filter lists
        cls.test_data = image

        return image['id']

    def _list_by_param_value_and_assert(self, params):
        """Perform list action with given params and validates result."""
        # Retrieve the list of images that meet the filter
        images_list = self.client.list_images(params=params)['images']
        # Validating params of fetched images
        msg = 'No images were found that met the filter criteria.'
        self.assertNotEmpty(images_list, msg)
        for image in images_list:
            for key in params:
                msg = "Failed to list images by %s" % key
                self.assertEqual(params[key], image[key], msg)

    def _list_sorted_by_image_size_and_assert(self, params, desc=False):
        """Validate an image list that has been sorted by size

        Perform list action with given params and validates the results are
        sorted by image size in either ascending or descending order.
        """
        # Retrieve the list of images that meet the filter
        images_list = self.client.list_images(params=params)['images']
        # Validate that the list was fetched sorted accordingly
        msg = 'No images were found that met the filter criteria.'
        self.assertNotEmpty(images_list, msg)
        sorted_list = [image['size'] for image in images_list
                       if image['size'] is not None]
        msg = 'The list of images was not sorted correctly.'
        self.assertEqual(sorted(sorted_list, reverse=desc), sorted_list, msg)

    @decorators.idempotent_id('1e341d7a-90a9-494c-b143-2cdf2aeb6aee')
    def test_list_no_params(self):
        """Simple test to see all fixture images returned"""
        images_list = self.client.list_images()['images']
        image_list = [image['id'] for image in images_list]

        for image in self.created_images:
            self.assertIn(image, image_list)

    @decorators.idempotent_id('9959ca1d-1aa7-4b7a-a1ea-0fff0499b37e')
    def test_list_images_param_container_format(self):
        """Test to get all images with a specific container_format"""
        params = {"container_format": self.test_data['container_format']}
        self._list_by_param_value_and_assert(params)

    @decorators.idempotent_id('4a4735a7-f22f-49b6-b0d9-66e1ef7453eb')
    def test_list_images_param_disk_format(self):
        """Test to get all images with disk_format = raw"""
        params = {"disk_format": "raw"}
        self._list_by_param_value_and_assert(params)

    @decorators.idempotent_id('7a95bb92-d99e-4b12-9718-7bc6ab73e6d2')
    def test_list_images_param_visibility(self):
        """Test to get all images with visibility = private"""
        params = {"visibility": "private"}
        self._list_by_param_value_and_assert(params)

    @decorators.idempotent_id('cf1b9a48-8340-480e-af7b-fe7e17690876')
    def test_list_images_param_size(self):
        """Test to get all images by size"""
        image_id = self.created_images[0]
        # Get image metadata
        image = self.client.show_image(image_id)

        params = {"size": image['size']}
        self._list_by_param_value_and_assert(params)

    @decorators.idempotent_id('4ad8c157-971a-4ba8-aa84-ed61154b1e7f')
    def test_list_images_param_min_max_size(self):
        """Test to get all images with min size and max size"""
        image_id = self.created_images[0]
        # Get image metadata
        image = self.client.show_image(image_id)

        size = image['size']
        params = {"size_min": size - 500, "size_max": size + 500}
        images_list = self.client.list_images(params=params)['images']
        image_size_list = map(lambda x: x['size'], images_list)

        for image_size in image_size_list:
            self.assertGreaterEqual(image_size, params['size_min'],
                                    "Failed to get images by size_min")
            self.assertLessEqual(image_size, params['size_max'],
                                 "Failed to get images by size_max")

    @decorators.idempotent_id('7fc9e369-0f58-4d05-9aa5-0969e2d59d15')
    def test_list_images_param_status(self):
        """Test to get all active images"""
        params = {"status": "active"}
        self._list_by_param_value_and_assert(params)

    @decorators.idempotent_id('e914a891-3cc8-4b40-ad32-e0a39ffbddbb')
    def test_list_images_param_limit(self):
        """Test to get images by limit"""
        params = {"limit": 1}
        images_list = self.client.list_images(params=params)['images']

        self.assertEqual(len(images_list), params['limit'],
                         "Failed to get images by limit")

    @decorators.idempotent_id('e9a44b91-31c8-4b40-a332-e0a39ffb4dbb')
    def test_list_image_param_owner(self):
        """Test to get images by owner"""
        image_id = self.created_images[0]
        # Get image metadata
        image = self.client.show_image(image_id)

        params = {"owner": image['owner']}
        self._list_by_param_value_and_assert(params)

    @decorators.idempotent_id('55c8f5f5-bfed-409d-a6d5-4caeda985d7b')
    def test_list_images_param_name(self):
        """Test to get images by name"""
        params = {'name': self.test_data['name']}
        self._list_by_param_value_and_assert(params)

    @decorators.idempotent_id('aa8ac4df-cff9-418b-8d0f-dd9c67b072c9')
    def test_list_images_param_tag(self):
        """Test to get images matching a tag"""
        params = {'tag': self.test_data['tags'][0]}
        images_list = self.client.list_images(params=params)['images']
        # Validating properties of fetched images
        self.assertNotEmpty(images_list)
        for image in images_list:
            msg = ("The image {image_name} does not have the expected tag "
                   "{expected_tag} among its tags: {observerd_tags}."
                   .format(image_name=image['name'],
                           expected_tag=self.test_data['tags'][0],
                           observerd_tags=image['tags']))
            self.assertIn(self.test_data['tags'][0], image['tags'], msg)

    @decorators.idempotent_id('eeadce49-04e0-43b7-aec7-52535d903e7a')
    def test_list_images_param_sort(self):
        """Test listing images sorting in descending order"""
        params = {'sort': 'size:desc'}
        self._list_sorted_by_image_size_and_assert(params, desc=True)

    @decorators.idempotent_id('9faaa0c2-c3a5-43e1-8f61-61c54b409a49')
    def test_list_images_param_sort_key_dir(self):
        """Test listing images sorting by size in descending order"""
        params = {'sort_key': 'size', 'sort_dir': 'desc'}
        self._list_sorted_by_image_size_and_assert(params, desc=True)

    @decorators.idempotent_id('622b925c-479f-4736-860d-adeaf13bc371')
    def test_get_image_schema(self):
        """Test to get image schema"""
        schema = "image"
        body = self.schemas_client.show_schema(schema)
        self.assertEqual("image", body['name'])

    @decorators.idempotent_id('25c8d7b2-df21-460f-87ac-93130bcdc684')
    def test_get_images_schema(self):
        """Test to get images schema"""
        schema = "images"
        body = self.schemas_client.show_schema(schema)
        self.assertEqual("images", body['name'])


class ListSharedImagesTest(base.BaseV2ImageTest):
    """Here we test the listing of a shared image information"""

    credentials = ['primary', 'alt']

    @classmethod
    def setup_clients(cls):
        super(ListSharedImagesTest, cls).setup_clients()
        cls.image_member_client = cls.os_primary.image_member_client_v2
        cls.alt_img_client = cls.os_alt.image_client_v2

    @decorators.idempotent_id('3fa50be4-8e38-4c02-a8db-7811bb780122')
    def test_list_images_param_member_status(self):
        """Test listing images by member_status and visibility"""
        # Create an image to be shared using default visibility
        image_file = io.BytesIO(data_utils.random_bytes(2048))
        container_format = CONF.image.container_formats[0]
        disk_format = CONF.image.disk_formats[0]
        image = self.create_image(container_format=container_format,
                                  disk_format=disk_format)
        self.client.store_image_file(image['id'], data=image_file)

        # Share the image created with the alt user
        self.image_member_client.create_image_member(
            image_id=image['id'], member=self.alt_img_client.tenant_id)

        # As an image consumer you need to provide the member_status parameter
        # along with the visibility=shared parameter in order for it to show
        # results
        params = {'member_status': 'pending', 'visibility': 'shared'}
        fetched_images = self.alt_img_client.list_images(params)['images']
        self.assertEqual(1, len(fetched_images))
        self.assertEqual(image['id'], fetched_images[0]['id'])



# Copyright 2015 IBM Corp.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy

import fixtures
from oslo_serialization import jsonutils as json

from tempest.api.compute import api_microversion_fixture
from tempest.lib.services.compute import flavors_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib import fake_http
from tempest.tests.lib.services import base


class TestFlavorsClient(base.BaseServiceTest):

    FAKE_FLAVOR = {
        "disk": 1,
        "id": "1",
        "links": [{
            "href": "http://openstack.example.com/v2/openstack/flavors/1",
            "rel": "self"}, {
            "href": "http://openstack.example.com/openstack/flavors/1",
            "rel": "bookmark"}],
        "name": "m1.tiny",
        "ram": 512,
        "swap": 1,
        "vcpus": 1
    }

    FAKE_FLAVOR_UPDATE = {
        "disk": 1,
        "id": "1",
        "links": [{
            "href": "http://openstack.example.com/v2/openstack/flavors/1",
            "rel": "self"}, {
            "href": "http://openstack.example.com/openstack/flavors/1",
            "rel": "bookmark"}],
        "name": "m1.tiny",
        "ram": 512,
        "swap": 1,
        "vcpus": 1,
        "description": 'new'
    }

    EXTRA_SPECS = {"extra_specs": {
        "key1": "value1",
        "key2": "value2"}
    }

    FAKE_FLAVOR_ACCESS = {
        "flavor_id": "10",
        "tenant_id": "1a951d988e264818afe520e78697dcbf"
    }

    def setUp(self):
        super(TestFlavorsClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.client = flavors_client.FlavorsClient(fake_auth,
                                                   'compute', 'regionOne')

    def _test_list_flavors(self, bytes_body=False):
        flavor = copy.deepcopy(TestFlavorsClient.FAKE_FLAVOR)
        # Remove extra attributes
        for attribute in ('disk', 'vcpus', 'ram', 'swap'):
            del flavor[attribute]
        expected = {'flavors': [flavor]}
        self.check_service_client_function(
            self.client.list_flavors,
            'tempest.lib.common.rest_client.RestClient.get',
            expected,
            bytes_body)

    def test_list_flavors_str_body(self):
        self._test_list_flavors(bytes_body=False)

    def test_list_flavors_byte_body(self):
        self._test_list_flavors(bytes_body=True)

    def _test_show_flavor(self, bytes_body=False):
        expected = {"flavor": TestFlavorsClient.FAKE_FLAVOR}
        self.check_service_client_function(
            self.client.show_flavor,
            'tempest.lib.common.rest_client.RestClient.get',
            expected,
            bytes_body,
            flavor_id='fake-id')

    def test_show_flavor_str_body(self):
        self._test_show_flavor(bytes_body=False)

    def test_show_flavor_byte_body(self):
        self._test_show_flavor(bytes_body=True)

    def _test_create_flavor(self, bytes_body=False):
        expected = {"flavor": TestFlavorsClient.FAKE_FLAVOR}
        request = copy.deepcopy(TestFlavorsClient.FAKE_FLAVOR)
        # The 'links' parameter should not be passed in
        del request['links']
        self.check_service_client_function(
            self.client.create_flavor,
            'tempest.lib.common.rest_client.RestClient.post',
            expected,
            bytes_body,
            **request)

    def test_create_flavor_str_body(self):
        self._test_create_flavor(bytes_body=False)

    def test_create_flavor__byte_body(self):
        self._test_create_flavor(bytes_body=True)

    def _test_update_flavor(self, bytes_body=False):
        self.useFixture(api_microversion_fixture.APIMicroversionFixture(
            '2.55'))
        expected = {"flavor": TestFlavorsClient.FAKE_FLAVOR_UPDATE}
        request = {"flavor": {"description": "updated description"}}
        self.check_service_client_function(
            self.client.update_flavor,
            'tempest.lib.common.rest_client.RestClient.put',
            expected,
            bytes_body,
            flavor_id='8c7aae5a-d315-4216-875b-ed9b6a5bcfc6',
            **request)

    def test_update_flavor_str_body(self):
        self._test_update_flavor(bytes_body=False)

    def test_update_flavor__byte_body(self):
        self._test_update_flavor(bytes_body=True)

    def test_delete_flavor(self):
        self.check_service_client_function(
            self.client.delete_flavor,
            'tempest.lib.common.rest_client.RestClient.delete',
            {}, status=202, flavor_id='c782b7a9-33cd-45f0-b795-7f87f456408b')

    def _test_is_resource_deleted(self, flavor_id, is_deleted=True,
                                  bytes_body=False):
        body = json.dumps({'flavors': [TestFlavorsClient.FAKE_FLAVOR]})
        if bytes_body:
            body = body.encode('utf-8')
        response = fake_http.fake_http_response({}, status=200), body
        self.useFixture(fixtures.MockPatch(
            'tempest.lib.common.rest_client.RestClient.get',
            return_value=response))
        self.assertEqual(is_deleted,
                         self.client.is_resource_deleted(flavor_id))

    def test_is_resource_deleted_true_str_body(self):
        self._test_is_resource_deleted('2', bytes_body=False)

    def test_is_resource_deleted_true_byte_body(self):
        self._test_is_resource_deleted('2', bytes_body=True)

    def test_is_resource_deleted_false_str_body(self):
        self._test_is_resource_deleted('1', is_deleted=False, bytes_body=False)

    def test_is_resource_deleted_false_byte_body(self):
        self._test_is_resource_deleted('1', is_deleted=False, bytes_body=True)

    def _test_set_flavor_extra_spec(self, bytes_body=False):
        self.check_service_client_function(
            self.client.set_flavor_extra_spec,
            'tempest.lib.common.rest_client.RestClient.post',
            TestFlavorsClient.EXTRA_SPECS,
            bytes_body,
            flavor_id='8c7aae5a-d315-4216-875b-ed9b6a5bcfc6',
            **TestFlavorsClient.EXTRA_SPECS)

    def test_set_flavor_extra_spec_str_body(self):
        self._test_set_flavor_extra_spec(bytes_body=False)

    def test_set_flavor_extra_spec_byte_body(self):
        self._test_set_flavor_extra_spec(bytes_body=True)

    def _test_list_flavor_extra_specs(self, bytes_body=False):
        self.check_service_client_function(
            self.client.list_flavor_extra_specs,
            'tempest.lib.common.rest_client.RestClient.get',
            TestFlavorsClient.EXTRA_SPECS,
            bytes_body,
            flavor_id='8c7aae5a-d315-4216-875b-ed9b6a5bcfc6')

    def test_list_flavor_extra_specs_str_body(self):
        self._test_list_flavor_extra_specs(bytes_body=False)

    def test_list_flavor_extra_specs__byte_body(self):
        self._test_list_flavor_extra_specs(bytes_body=True)

    def _test_show_flavor_extra_spec(self, bytes_body=False):
        expected = {"key": "value"}
        self.check_service_client_function(
            self.client.show_flavor_extra_spec,
            'tempest.lib.common.rest_client.RestClient.get',
            expected,
            bytes_body,
            flavor_id='8c7aae5a-d315-4216-875b-ed9b6a5bcfc6',
            key='key')

    def test_show_flavor_extra_spec_str_body(self):
        self._test_show_flavor_extra_spec(bytes_body=False)

    def test_show_flavor_extra_spec__byte_body(self):
        self._test_show_flavor_extra_spec(bytes_body=True)

    def _test_update_flavor_extra_spec(self, bytes_body=False):
        expected = {"key1": "value"}
        self.check_service_client_function(
            self.client.update_flavor_extra_spec,
            'tempest.lib.common.rest_client.RestClient.put',
            expected,
            bytes_body,
            flavor_id='8c7aae5a-d315-4216-875b-ed9b6a5bcfc6',
            key='key1', **expected)

    def test_update_flavor_extra_spec_str_body(self):
        self._test_update_flavor_extra_spec(bytes_body=False)

    def test_update_flavor_extra_spec_byte_body(self):
        self._test_update_flavor_extra_spec(bytes_body=True)

    def test_unset_flavor_extra_spec(self):
        self.check_service_client_function(
            self.client.unset_flavor_extra_spec,
            'tempest.lib.common.rest_client.RestClient.delete', {},
            flavor_id='c782b7a9-33cd-45f0-b795-7f87f456408b', key='key')

    def _test_list_flavor_access(self, bytes_body=False):
        expected = {'flavor_access': [TestFlavorsClient.FAKE_FLAVOR_ACCESS]}
        self.check_service_client_function(
            self.client.list_flavor_access,
            'tempest.lib.common.rest_client.RestClient.get',
            expected,
            bytes_body,
            flavor_id='8c7aae5a-d315-4216-875b-ed9b6a5bcfc6')

    def test_list_flavor_access_str_body(self):
        self._test_list_flavor_access(bytes_body=False)

    def test_list_flavor_access_byte_body(self):
        self._test_list_flavor_access(bytes_body=True)

    def _test_add_flavor_access(self, bytes_body=False):
        expected = {
            "flavor_access": [TestFlavorsClient.FAKE_FLAVOR_ACCESS]
        }
        self.check_service_client_function(
            self.client.add_flavor_access,
            'tempest.lib.common.rest_client.RestClient.post',
            expected,
            bytes_body,
            flavor_id='8c7aae5a-d315-4216-875b-ed9b6a5bcfc6',
            tenant_id='1a951d988e264818afe520e78697dcbf')

    def test_add_flavor_access_str_body(self):
        self._test_add_flavor_access(bytes_body=False)

    def test_add_flavor_access_byte_body(self):
        self._test_add_flavor_access(bytes_body=True)

    def _test_remove_flavor_access(self, bytes_body=False):
        expected = {
            "flavor_access": [TestFlavorsClient.FAKE_FLAVOR_ACCESS]
        }
        self.check_service_client_function(
            self.client.remove_flavor_access,
            'tempest.lib.common.rest_client.RestClient.post',
            expected,
            bytes_body,
            flavor_id='10',
            tenant_id='a6edd4d66ad04245b5d2d8716ecc91e3')

    def test_remove_flavor_access_str_body(self):
        self._test_remove_flavor_access(bytes_body=False)

    def test_remove_flavor_access_byte_body(self):
        self._test_remove_flavor_access(bytes_body=True)



# Copyright 2013 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import netaddr
import testtools

from tempest.api.compute import base
from tempest.common import utils
from tempest import config
from tempest.lib import decorators
from tempest.lib import exceptions

CONF = config.CONF


# TODO(mriedem): Remove this test class once the nova queens branch goes into
# extended maintenance mode.
class VirtualInterfacesTestJSON(base.BaseV2ComputeTest):
    """Test virtual interfaces API with compute microversion less than 2.44"""

    max_microversion = '2.43'

    depends_on_nova_network = True

    create_default_network = True

    @classmethod
    def setup_clients(cls):
        super(VirtualInterfacesTestJSON, cls).setup_clients()
        cls.client = cls.servers_client

    @classmethod
    def resource_setup(cls):
        super(VirtualInterfacesTestJSON, cls).resource_setup()
        cls.server = cls.create_test_server(wait_until='ACTIVE')

    @decorators.idempotent_id('96c4e2ef-5e4d-4d7f-87f5-fed6dca18016')
    @utils.services('network')
    def test_list_virtual_interfaces(self):
        """Test listing virtual interfaces of a server"""
        if CONF.service_available.neutron:
            with testtools.ExpectedException(exceptions.BadRequest):
                self.client.list_virtual_interfaces(self.server['id'])
        else:
            output = self.client.list_virtual_interfaces(self.server['id'])
            virt_ifaces = output['virtual_interfaces']
            self.assertNotEmpty(virt_ifaces,
                                'Expected virtual interfaces, got 0 '
                                'interfaces.')
            for virt_iface in virt_ifaces:
                mac_address = virt_iface['mac_address']
                self.assertTrue(netaddr.valid_mac(mac_address),
                                "Invalid mac address detected. mac address: %s"
                                % mac_address)



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import time

from testtools import matchers

from tempest.api.identity import base
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators


class UsersTestJSON(base.BaseIdentityV2AdminTest):
    """Test keystone users via v2 API"""

    @classmethod
    def resource_setup(cls):
        super(UsersTestJSON, cls).resource_setup()
        cls.alt_user = data_utils.rand_name('test_user')
        cls.alt_email = cls.alt_user + '@testmail.tm'

    @decorators.attr(type='smoke')
    @decorators.idempotent_id('2d55a71e-da1d-4b43-9c03-d269fd93d905')
    def test_create_user(self):
        """Test creating a user via v2 API"""
        tenant = self.setup_test_tenant()
        user = self.create_test_user(name=self.alt_user, tenantId=tenant['id'])
        self.assertEqual(self.alt_user, user['name'])

    @decorators.idempotent_id('89d9fdb8-15c2-4304-a429-48715d0af33d')
    def test_create_user_with_enabled(self):
        """Test creating a user with enabled : False via v2 API"""
        tenant = self.setup_test_tenant()
        name = data_utils.rand_name('test_user')
        user = self.create_test_user(name=name,
                                     tenantId=tenant['id'],
                                     email=self.alt_email,
                                     enabled=False)
        self.assertEqual(name, user['name'])
        self.assertEqual(False, user['enabled'])
        self.assertEqual(self.alt_email, user['email'])

    @decorators.idempotent_id('39d05857-e8a5-4ed4-ba83-0b52d3ab97ee')
    def test_update_user(self):
        """Test updating user attributes via v2 API"""
        tenant = self.setup_test_tenant()
        user = self.create_test_user(tenantId=tenant['id'])

        # Updating user details with new values
        u_name2 = data_utils.rand_name('user2')
        u_email2 = u_name2 + '@testmail.tm'
        update_user = self.users_client.update_user(user['id'], name=u_name2,
                                                    email=u_email2,
                                                    enabled=False)['user']
        self.assertEqual(u_name2, update_user['name'])
        self.assertEqual(u_email2, update_user['email'])
        self.assertEqual(False, update_user['enabled'])
        # GET by id after updating
        updated_user = self.users_client.show_user(user['id'])['user']
        # Assert response body of GET after updating
        self.assertEqual(u_name2, updated_user['name'])
        self.assertEqual(u_email2, updated_user['email'])
        self.assertEqual(False, update_user['enabled'])

    @decorators.idempotent_id('29ed26f4-a74e-4425-9a85-fdb49fa269d2')
    def test_delete_user(self):
        """Test deleting a user via v2 API"""
        tenant = self.setup_test_tenant()
        user = self.create_test_user(tenantId=tenant['id'])
        self.users_client.delete_user(user['id'])

    @decorators.idempotent_id('aca696c3-d645-4f45-b728-63646045beb1')
    def test_user_authentication(self):
        """Test that valid user's token is authenticated via v2 API"""
        password = data_utils.rand_password()
        user = self.setup_test_user(password)
        tenant = self.tenants_client.show_tenant(user['tenantId'])['tenant']
        # Get a token
        self.token_client.auth(user['name'],
                               password,
                               tenant['name'])
        # Re-auth
        self.token_client.auth(user['name'],
                               password,
                               tenant['name'])

    @decorators.idempotent_id('5d1fa498-4c2d-4732-a8fe-2b054598cfdd')
    def test_authentication_request_without_token(self):
        """Test authentication request without token via v2 API"""
        # Request for token authentication with a valid token in header
        password = data_utils.rand_password()
        user = self.setup_test_user(password)
        tenant = self.tenants_client.show_tenant(user['tenantId'])['tenant']
        self.token_client.auth(user['name'],
                               password,
                               tenant['name'])
        # Get the token of the current client
        token = self.client.auth_provider.get_token()
        # Delete the token from database
        self.client.delete_token(token)
        # Re-auth
        self.token_client.auth(user['name'],
                               password,
                               tenant['name'])
        self.client.auth_provider.clear_auth()

    @decorators.idempotent_id('a149c02e-e5e0-4b89-809e-7e8faf33ccda')
    def test_get_users(self):
        """Test getting users via v2 API

        Get a list of users and find the test user
        """
        user = self.setup_test_user()
        users = self.users_client.list_users()['users']
        self.assertThat([u['name'] for u in users],
                        matchers.Contains(user['name']),
                        "Could not find %s" % user['name'])

    @decorators.idempotent_id('6e317209-383a-4bed-9f10-075b7c82c79a')
    def test_list_users_for_tenant(self):
        """Test returning a list of all users for a tenant via v2 API"""
        tenant = self.setup_test_tenant()
        user_ids = list()
        fetched_user_ids = list()
        user1 = self.create_test_user(tenantId=tenant['id'])
        user_ids.append(user1['id'])
        user2 = self.create_test_user(tenantId=tenant['id'])
        user_ids.append(user2['id'])
        # List of users for the respective tenant ID
        body = (self.tenants_client.list_tenant_users(tenant['id'])
                ['users'])
        for i in body:
            fetched_user_ids.append(i['id'])
        # verifying the user Id in the list
        missing_users =\
            [user for user in user_ids if user not in fetched_user_ids]
        self.assertEmpty(missing_users,
                         "Failed to find user %s in fetched list" %
                         ', '.join(m_user for m_user in missing_users))

    @decorators.idempotent_id('a8b54974-40e1-41c0-b812-50fc90827971')
    def test_list_users_with_roles_for_tenant(self):
        """Test listing users on tenant with roles assigned via v2 API"""
        user = self.setup_test_user()
        tenant = self.tenants_client.show_tenant(user['tenantId'])['tenant']
        role = self.setup_test_role()
        # Assigning roles to two users
        user_ids = list()
        fetched_user_ids = list()
        user_ids.append(user['id'])
        role = self.roles_client.create_user_role_on_project(
            tenant['id'], user['id'], role['id'])['role']

        second_user = self.create_test_user(tenantId=tenant['id'])
        user_ids.append(second_user['id'])
        role = self.roles_client.create_user_role_on_project(
            tenant['id'], second_user['id'], role['id'])['role']
        # List of users with roles for the respective tenant ID
        body = (self.tenants_client.list_tenant_users(tenant['id'])['users'])
        for i in body:
            fetched_user_ids.append(i['id'])
        # verifying the user Id in the list
        missing_users = [missing_user for missing_user in user_ids
                         if missing_user not in fetched_user_ids]
        self.assertEmpty(missing_users,
                         "Failed to find user %s in fetched list" %
                         ', '.join(m_user for m_user in missing_users))

    @decorators.idempotent_id('1aeb25ac-6ec5-4d8b-97cb-7ac3567a989f')
    def test_update_user_password(self):
        """Test updating of user password via v2 API"""
        user = self.setup_test_user()
        tenant = self.tenants_client.show_tenant(user['tenantId'])['tenant']
        # Updating the user with new password
        new_pass = data_utils.rand_password()
        update_user = self.users_client.update_user_password(
            user['id'], password=new_pass)['user']
        self.assertEqual(update_user['id'], user['id'])
        # NOTE(morganfainberg): Fernet tokens are not subsecond aware and
        # Keystone should only be precise to the second. Sleep to ensure
        # we are passing the second boundary.
        time.sleep(1)
        # Validate the updated password through getting a token.
        body = self.token_client.auth(user['name'], new_pass,
                                      tenant['name'])
        self.assertIn('id', body['token'])



# Copyright 2016 NEC Corporation.  All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from tempest.lib.services.volume.v3 import quotas_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestQuotasClient(base.BaseServiceTest):
    FAKE_QUOTAS = {
        "quota_set": {
            "id": '730a1cbd-68ca-4d68-8e09-d603f2dfa72b',
            "gigabytes": 5,
            "snapshots": 10,
            "volumes": 20,
            'backups': 10,
            'groups': 10,
            'per_volume_gigabytes': 1000,
            'backup_gigabytes': 2000
        }
    }

    FAKE_UPDATE_QUOTAS_RESPONSE = {
        "quota_set": {
            "gigabytes": 6,
            "snapshots": 11,
            "volumes": 21,
            'backups': 11,
            'groups': 11,
            'per_volume_gigabytes': 1001,
            'backup_gigabytes': 2001
        }
    }

    def setUp(self):
        super(TestQuotasClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.client = quotas_client.QuotasClient(fake_auth,
                                                 'volume',
                                                 'regionOne')

    def _test_show_default_quota_set(self, bytes_body=False):
        self.check_service_client_function(
            self.client.show_default_quota_set,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_QUOTAS,
            bytes_body, tenant_id="fake_tenant")

    def _test_show_quota_set(self, bytes_body=False):
        self.check_service_client_function(
            self.client.show_quota_set,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_QUOTAS,
            bytes_body, tenant_id="fake_tenant")

    def _test_update_quota_set(self, bytes_body=False):
        self.check_service_client_function(
            self.client.update_quota_set,
            'tempest.lib.common.rest_client.RestClient.put',
            self.FAKE_UPDATE_QUOTAS_RESPONSE,
            bytes_body, tenant_id="fake_tenant")

    def test_show_default_quota_set_with_str_body(self):
        self._test_show_default_quota_set()

    def test_show_default_quota_set_with_bytes_body(self):
        self._test_show_default_quota_set(bytes_body=True)

    def test_show_quota_set_with_str_body(self):
        self._test_show_quota_set()

    def test_show_quota_set_with_bytes_body(self):
        self._test_show_quota_set(bytes_body=True)

    def test_update_quota_set_with_str_body(self):
        self._test_update_quota_set()

    def test_update_quota_set_with_bytes_body(self):
        self._test_update_quota_set(bytes_body=True)

    def test_delete_quota_set(self):
        self.check_service_client_function(
            self.client.delete_quota_set,
            'tempest.lib.common.rest_client.RestClient.delete',
            {},
            tenant_id="fake_tenant")



# Copyright 2013 Huawei Technologies Co.,LTD.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.identity import base
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc


class RolesNegativeTestJSON(base.BaseIdentityV2AdminTest):
    """Negative tests of keystone roles via v2 API"""

    def _get_role_params(self):
        user = self.setup_test_user()
        tenant = self.tenants_client.show_tenant(user['tenantId'])['tenant']
        role = self.setup_test_role()
        return (user, tenant, role)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('d5d5f1df-f8ca-4de0-b2ef-259c1cc67025')
    def test_list_roles_by_unauthorized_user(self):
        """Test Non-admin user should not be able to list roles via v2 API"""
        self.assertRaises(lib_exc.Forbidden,
                          self.non_admin_roles_client.list_roles)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('11a3c7da-df6c-40c2-abc2-badd682edf9f')
    def test_list_roles_request_without_token(self):
        """Test listing roles without a valid token via v2 API should fail"""
        token = self.client.auth_provider.get_token()
        self.client.delete_token(token)
        self.assertRaises(lib_exc.Unauthorized, self.roles_client.list_roles)
        self.client.auth_provider.clear_auth()

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('c0b89e56-accc-4c73-85f8-9c0f866104c1')
    def test_role_create_blank_name(self):
        """Test creating a role with a blank name via v2 API is not allowed"""
        self.assertRaises(lib_exc.BadRequest, self.roles_client.create_role,
                          name='')

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('585c8998-a8a4-4641-a5dd-abef7a8ced00')
    def test_create_role_by_unauthorized_user(self):
        """Test non-admin user should not be able to create role via v2 API"""
        role_name = data_utils.rand_name(name='role')
        self.assertRaises(lib_exc.Forbidden,
                          self.non_admin_roles_client.create_role,
                          name=role_name)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('a7edd17a-e34a-4aab-8bb7-fa6f498645b8')
    def test_create_role_request_without_token(self):
        """Test creating role without a valid token via v2 API should fail"""
        token = self.client.auth_provider.get_token()
        self.client.delete_token(token)
        role_name = data_utils.rand_name(name='role')
        self.assertRaises(lib_exc.Unauthorized,
                          self.roles_client.create_role, name=role_name)
        self.client.auth_provider.clear_auth()

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('c0cde2c8-81c1-4bb0-8fe2-cf615a3547a8')
    def test_role_create_duplicate(self):
        """Test role names should be unique via v2 API"""
        role_name = data_utils.rand_name(name='role-dup')
        body = self.roles_client.create_role(name=role_name)['role']
        role1_id = body.get('id')
        self.addCleanup(self.roles_client.delete_role, role1_id)
        self.assertRaises(lib_exc.Conflict, self.roles_client.create_role,
                          name=role_name)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('15347635-b5b1-4a87-a280-deb2bd6d865e')
    def test_delete_role_by_unauthorized_user(self):
        """Test non-admin user should not be able to delete role via v2 API"""
        role_name = data_utils.rand_name(name='role')
        body = self.roles_client.create_role(name=role_name)['role']
        self.addCleanup(self.roles_client.delete_role, body['id'])
        role_id = body.get('id')
        self.assertRaises(lib_exc.Forbidden,
                          self.non_admin_roles_client.delete_role, role_id)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('44b60b20-70de-4dac-beaf-a3fc2650a16b')
    def test_delete_role_request_without_token(self):
        """Test deleting role without a valid token via v2 API should fail"""
        role_name = data_utils.rand_name(name='role')
        body = self.roles_client.create_role(name=role_name)['role']
        self.addCleanup(self.roles_client.delete_role, body['id'])
        role_id = body.get('id')
        token = self.client.auth_provider.get_token()
        self.client.delete_token(token)
        self.assertRaises(lib_exc.Unauthorized,
                          self.roles_client.delete_role,
                          role_id)
        self.client.auth_provider.clear_auth()

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('38373691-8551-453a-b074-4260ad8298ef')
    def test_delete_role_non_existent(self):
        """Test deleting a non existent role via v2 API should fail"""
        non_existent_role = data_utils.rand_uuid_hex()
        self.assertRaises(lib_exc.NotFound, self.roles_client.delete_role,
                          non_existent_role)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('391df5cf-3ec3-46c9-bbe5-5cb58dd4dc41')
    def test_assign_user_role_by_unauthorized_user(self):
        """Test non-admin user assigning a role to user via v2 API

        Non-admin user should not be authorized to assign a role to user via
        v2 API.
        """
        (user, tenant, role) = self._get_role_params()
        self.assertRaises(
            lib_exc.Forbidden,
            self.non_admin_roles_client.create_user_role_on_project,
            tenant['id'], user['id'], role['id'])

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('f0d2683c-5603-4aee-95d7-21420e87cfd8')
    def test_assign_user_role_request_without_token(self):
        """Test assigning a role to a user without a valid token via v2 API

        Assigning a role to a user without a valid token via v2 API should
        fail.
        """
        (user, tenant, role) = self._get_role_params()
        token = self.client.auth_provider.get_token()
        self.client.delete_token(token)
        self.assertRaises(
            lib_exc.Unauthorized,
            self.roles_client.create_user_role_on_project, tenant['id'],
            user['id'], role['id'])
        self.client.auth_provider.clear_auth()

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('99b297f6-2b5d-47c7-97a9-8b6bb4f91042')
    def test_assign_user_role_for_non_existent_role(self):
        """Test assigning a non existent role to user via v2 API

        Assigning a non existent role to user via v2 API should fail.
        """
        (user, tenant, _) = self._get_role_params()
        non_existent_role = data_utils.rand_uuid_hex()
        self.assertRaises(lib_exc.NotFound,
                          self.roles_client.create_user_role_on_project,
                          tenant['id'], user['id'], non_existent_role)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('b2285aaa-9e76-4704-93a9-7a8acd0a6c8f')
    def test_assign_user_role_for_non_existent_tenant(self):
        """Test assigning a role on a non existent tenant via v2 API

        Assigning a role on a non existent tenant via v2 API should fail.
        """
        (user, _, role) = self._get_role_params()
        non_existent_tenant = data_utils.rand_uuid_hex()
        self.assertRaises(lib_exc.NotFound,
                          self.roles_client.create_user_role_on_project,
                          non_existent_tenant, user['id'], role['id'])

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('5c3132cd-c4c8-4402-b5ea-71eb44e97793')
    def test_assign_duplicate_user_role(self):
        """Test duplicate user role should not get assigned via v2 API"""
        (user, tenant, role) = self._get_role_params()
        self.roles_client.create_user_role_on_project(tenant['id'],
                                                      user['id'],
                                                      role['id'])
        self.assertRaises(lib_exc.Conflict,
                          self.roles_client.create_user_role_on_project,
                          tenant['id'], user['id'], role['id'])

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('d0537987-0977-448f-a435-904c15de7298')
    def test_remove_user_role_by_unauthorized_user(self):
        """Test non-admin user removing a user's role via v2 API

        Non-admin user should not be authorized to remove a user's role via
        v2 API
        """
        (user, tenant, role) = self._get_role_params()
        self.roles_client.create_user_role_on_project(tenant['id'],
                                                      user['id'],
                                                      role['id'])
        self.assertRaises(
            lib_exc.Forbidden,
            self.non_admin_roles_client.delete_role_from_user_on_project,
            tenant['id'], user['id'], role['id'])

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('cac81cf4-c1d2-47dc-90d3-f2b7eb572286')
    def test_remove_user_role_request_without_token(self):
        """Test removing a user's role without a valid token via v2 API

        Removing a user's role without a valid token via v2 API should fail.
        """
        (user, tenant, role) = self._get_role_params()
        self.roles_client.create_user_role_on_project(tenant['id'],
                                                      user['id'],
                                                      role['id'])
        token = self.client.auth_provider.get_token()
        self.client.delete_token(token)
        self.assertRaises(lib_exc.Unauthorized,
                          self.roles_client.delete_role_from_user_on_project,
                          tenant['id'], user['id'], role['id'])
        self.client.auth_provider.clear_auth()

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('ab32d759-cd16-41f1-a86e-44405fa9f6d2')
    def test_remove_user_role_non_existent_role(self):
        """Test deleting a non existent role from a user via v2 API

        Deleting a non existent role from a user via v2 API should fail.
        """
        (user, tenant, role) = self._get_role_params()
        self.roles_client.create_user_role_on_project(tenant['id'],
                                                      user['id'],
                                                      role['id'])
        non_existent_role = data_utils.rand_uuid_hex()
        self.assertRaises(lib_exc.NotFound,
                          self.roles_client.delete_role_from_user_on_project,
                          tenant['id'], user['id'], non_existent_role)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('67a679ec-03dd-4551-bbfc-d1c93284f023')
    def test_remove_user_role_non_existent_tenant(self):
        """Test removing a role from a non existent tenant via v2 API

        Removing a role from a non existent tenant via v2 API should fail.
        """
        (user, tenant, role) = self._get_role_params()
        self.roles_client.create_user_role_on_project(tenant['id'],
                                                      user['id'],
                                                      role['id'])
        non_existent_tenant = data_utils.rand_uuid_hex()
        self.assertRaises(lib_exc.NotFound,
                          self.roles_client.delete_role_from_user_on_project,
                          non_existent_tenant, user['id'], role['id'])

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('7391ab4c-06f3-477a-a64a-c8e55ce89837')
    def test_list_user_roles_by_unauthorized_user(self):
        """Test non-admin user listing a user's roles via v2 API

        Non-admin user should not be authorized to list a user's roles via v2
        API.
        """
        (user, tenant, role) = self._get_role_params()
        self.roles_client.create_user_role_on_project(tenant['id'],
                                                      user['id'],
                                                      role['id'])
        self.assertRaises(
            lib_exc.Forbidden,
            self.non_admin_roles_client.list_user_roles_on_project,
            tenant['id'], user['id'])

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('682adfb2-fd5f-4b0a-a9ca-322e9bebb907')
    def test_list_user_roles_request_without_token(self):
        """Test listing user's roles without a valid token via v2 API

        Listing user's roles without a valid token via v2 API should fail
        """
        (user, tenant, _) = self._get_role_params()
        token = self.client.auth_provider.get_token()
        self.client.delete_token(token)
        try:
            self.assertRaises(lib_exc.Unauthorized,
                              self.roles_client.list_user_roles_on_project,
                              tenant['id'],
                              user['id'])
        finally:
            self.client.auth_provider.clear_auth()



# Copyright 2017 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.services.volume.v3 import scheduler_stats_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestSchedulerStatsClient(base.BaseServiceTest):
    FAKE_POOLS_LIST = {
        "pools": [
            {
                "name": "pool1",
                "capabilities": {
                    "updated": "2014-10-28T00:00:00-00:00",
                    "total_capacity_gb": 1024,
                    "free_capacity_gb": 100,
                    "volume_backend_name": "pool1",
                    "reserved_percentage": 0,
                    "driver_version": "1.0.0",
                    "timestamp": "2014-10-28T00:00:00-00:00",
                    "storage_protocol": "iSCSI",
                    "vendor_name": "vendor",
                    "QoS_support": False
                }
            },
            {
                "name": "pool2",
                "capabilities": {
                    "updated": "2014-10-28T00:00:00-00:00",
                    "total_capacity_gb": 512,
                    "free_capacity_gb": 200,
                    "volume_backend_name": "pool2",
                    "reserved_percentage": 0,
                    "driver_version": "1.0.2",
                    "timestamp": "2014-10-28T00:00:00-00:00",
                    "storage_protocol": "iSER",
                    "vendor_name": "vendor",
                    "QoS_support": True
                }
            }
        ]
    }

    def setUp(self):
        super(TestSchedulerStatsClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.client = scheduler_stats_client.SchedulerStatsClient(
            fake_auth, 'volume', 'regionOne')

    def _test_list_pools(self, bytes_body=False, detail=False):
        resp_body = []
        if detail:
            resp_body = self.FAKE_POOLS_LIST
        else:
            resp_body = {'pools': [{'name': pool['name']}
                                   for pool in self.FAKE_POOLS_LIST['pools']]}
        self.check_service_client_function(
            self.client.list_pools,
            'tempest.lib.common.rest_client.RestClient.get',
            resp_body,
            bytes_body,
            detail=detail)

    def test_list_pools_with_str_body(self):
        self._test_list_pools()

    def test_list_pools_with_str_body_and_detail(self):
        self._test_list_pools(detail=True)

    def test_list_pools_with_bytes_body(self):
        self._test_list_pools(bytes_body=True)

    def test_list_pools_with_bytes_body_and_detail(self):
        self._test_list_pools(bytes_body=True, detail=True)



# Copyright 2016 EasyStack.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from urllib import parse as urllib

from oslo_serialization import jsonutils as json

from tempest.lib.common import rest_client


class NamespaceObjectsClient(rest_client.RestClient):
    api_version = "v2"

    def list_namespace_objects(self, namespace, **kwargs):
        """Lists all namespace objects.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/image/v2/metadefs-index.html#list-objects
        """
        url = 'metadefs/namespaces/%s/objects' % namespace
        if kwargs:
            url += '?%s' % urllib.urlencode(kwargs)
        resp, body = self.get(url)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def create_namespace_object(self, namespace, **kwargs):
        """Create a namespace object

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/image/v2/metadefs-index.html#create-object
        """
        url = 'metadefs/namespaces/%s/objects' % namespace
        data = json.dumps(kwargs)
        resp, body = self.post(url, data)
        self.expected_success(201, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def update_namespace_object(self, namespace, object_name, **kwargs):
        """Update a namespace object

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/image/v2/metadefs-index.html#update-object
        """
        url = 'metadefs/namespaces/%s/objects/%s' % (namespace, object_name)
        data = json.dumps(kwargs)
        resp, body = self.put(url, data)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def show_namespace_object(self, namespace, object_name):
        """Show a namespace object

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/image/v2/metadefs-index.html#show-object
        """
        url = 'metadefs/namespaces/%s/objects/%s' % (namespace, object_name)
        resp, body = self.get(url)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def delete_namespace_object(self, namespace, object_name):
        """Delete a namespace object

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/image/v2/metadefs-index.html#delete-object
        """
        url = 'metadefs/namespaces/%s/objects/%s' % (namespace, object_name)
        resp, _ = self.delete(url)
        self.expected_success(204, resp.status)
        return rest_client.ResponseBody(resp)



# Copyright 2015 Dell Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from urllib import parse as urllib

from oslo_log import log as logging

from tempest import clients
from tempest.common import credentials_factory as credentials
from tempest.common import identity
from tempest.common import utils
from tempest.common.utils import net_info
from tempest import config
from tempest.lib import exceptions

LOG = logging.getLogger('tempest.cmd.cleanup')
CONF = config.CONF

CONF_FLAVORS = None
CONF_IMAGES = None
CONF_NETWORKS = []
CONF_PRIV_NETWORK_NAME = None
CONF_PUB_NETWORK = None
CONF_PUB_ROUTER = None
CONF_PROJECTS = None
CONF_USERS = None

IS_CINDER = None
IS_GLANCE = None
IS_NEUTRON = None
IS_NOVA = None


def init_conf():
    global CONF_FLAVORS
    global CONF_IMAGES
    global CONF_NETWORKS
    global CONF_PRIV_NETWORK
    global CONF_PRIV_NETWORK_NAME
    global CONF_PUB_NETWORK
    global CONF_PUB_ROUTER
    global CONF_PROJECTS
    global CONF_USERS
    global IS_CINDER
    global IS_GLANCE
    global IS_HEAT
    global IS_NEUTRON
    global IS_NOVA

    IS_CINDER = CONF.service_available.cinder
    IS_GLANCE = CONF.service_available.glance
    IS_NEUTRON = CONF.service_available.neutron
    IS_NOVA = CONF.service_available.nova

    CONF_FLAVORS = [CONF.compute.flavor_ref, CONF.compute.flavor_ref_alt]
    CONF_IMAGES = [CONF.compute.image_ref, CONF.compute.image_ref_alt]
    CONF_PRIV_NETWORK_NAME = CONF.compute.fixed_network_name
    CONF_PUB_NETWORK = CONF.network.public_network_id
    CONF_PUB_ROUTER = CONF.network.public_router_id
    CONF_PROJECTS = [CONF.auth.admin_project_name]
    CONF_USERS = [CONF.auth.admin_username]

    if IS_NEUTRON:
        CONF_PRIV_NETWORK = _get_network_id(CONF.compute.fixed_network_name,
                                            CONF.auth.admin_project_name)
        CONF_NETWORKS = [CONF_PUB_NETWORK, CONF_PRIV_NETWORK]


def _get_network_id(net_name, project_name):
    am = clients.Manager(
        credentials.get_configured_admin_credentials())
    net_cl = am.networks_client
    pr_cl = am.projects_client

    networks = net_cl.list_networks()
    project = identity.get_project_by_name(pr_cl, project_name)
    p_id = project['id']
    n_id = None
    for net in networks['networks']:
        if (net['project_id'] == p_id and net['name'] == net_name):
            n_id = net['id']
            break
    return n_id


class BaseService(object):
    def __init__(self, kwargs):
        self.client = None
        for key, value in kwargs.items():
            setattr(self, key, value)

        self.tenant_filter = {}
        if hasattr(self, 'tenant_id'):
            self.tenant_filter['project_id'] = self.tenant_id

    def _filter_by_tenant_id(self, item_list):
        if (item_list is None or
                not item_list or
                not hasattr(self, 'tenant_id') or
                self.tenant_id is None or
                'tenant_id' not in item_list[0]):
            return item_list

        return [item for item in item_list
                if item['tenant_id'] == self.tenant_id]

    def list(self):
        pass

    def delete(self):
        pass

    def dry_run(self):
        pass

    def save_state(self):
        pass

    def run(self):
        try:
            if self.is_dry_run:
                self.dry_run()
            elif self.is_save_state:
                self.save_state()
            else:
                self.delete()
        except exceptions.NotImplemented as exc:
            # Many OpenStack services use extensions logic to implement the
            # features or resources. Tempest cleanup tries to clean up the test
            # resources without having much logic of extensions checks etc.
            # If any of the extension is missing then, service will return
            # NotImplemented error.
            msg = ("Got NotImplemented error in %s, full exception: %s" %
                   (str(self.__class__), str(exc)))
            LOG.exception(msg)
            self.got_exceptions.append(exc)


class SnapshotService(BaseService):

    def __init__(self, manager, **kwargs):
        super(SnapshotService, self).__init__(kwargs)
        self.client = manager.snapshots_client_latest

    def list(self):
        client = self.client
        snaps = client.list_snapshots()['snapshots']
        if not self.is_save_state:
            # recreate list removing saved snapshots
            snaps = [snap for snap in snaps if snap['id']
                     not in self.saved_state_json['snapshots'].keys()]
        LOG.debug("List count, %s Snapshots", len(snaps))
        return snaps

    def delete(self):
        snaps = self.list()
        client = self.client
        for snap in snaps:
            try:
                LOG.debug("Deleting Snapshot with id %s", snap['id'])
                client.delete_snapshot(snap['id'])
            except Exception:
                LOG.exception("Delete Snapshot %s exception.", snap['id'])

    def dry_run(self):
        snaps = self.list()
        self.data['snapshots'] = snaps

    def save_state(self):
        snaps = self.list()
        self.data['snapshots'] = {}
        for snap in snaps:
            self.data['snapshots'][snap['id']] = snap['name']


class ServerService(BaseService):
    def __init__(self, manager, **kwargs):
        super(ServerService, self).__init__(kwargs)
        self.client = manager.servers_client
        self.server_groups_client = manager.server_groups_client

    def list(self):
        client = self.client
        servers_body = client.list_servers()
        servers = servers_body['servers']
        if not self.is_save_state:
            # recreate list removing saved servers
            servers = [server for server in servers if server['id']
                       not in self.saved_state_json['servers'].keys()]
        LOG.debug("List count, %s Servers", len(servers))
        return servers

    def delete(self):
        client = self.client
        servers = self.list()
        for server in servers:
            try:
                LOG.debug("Deleting Server with id %s", server['id'])
                client.delete_server(server['id'])
            except Exception:
                LOG.exception("Delete Server %s exception.", server['id'])

    def dry_run(self):
        servers = self.list()
        self.data['servers'] = servers

    def save_state(self):
        servers = self.list()
        self.data['servers'] = {}
        for server in servers:
            self.data['servers'][server['id']] = server['name']


class ServerGroupService(ServerService):

    def list(self):
        client = self.server_groups_client
        sgs = client.list_server_groups()['server_groups']
        if not self.is_save_state:
            # recreate list removing saved server_groups
            sgs = [sg for sg in sgs if sg['id']
                   not in self.saved_state_json['server_groups'].keys()]
        LOG.debug("List count, %s Server Groups", len(sgs))
        return sgs

    def delete(self):
        client = self.server_groups_client
        sgs = self.list()
        for sg in sgs:
            try:
                LOG.debug("Deleting Server Group with id %s", sg['id'])
                client.delete_server_group(sg['id'])
            except Exception:
                LOG.exception("Delete Server Group %s exception.", sg['id'])

    def dry_run(self):
        sgs = self.list()
        self.data['server_groups'] = sgs

    def save_state(self):
        sgs = self.list()
        self.data['server_groups'] = {}
        for sg in sgs:
            self.data['server_groups'][sg['id']] = sg['name']


class KeyPairService(BaseService):
    def __init__(self, manager, **kwargs):
        super(KeyPairService, self).__init__(kwargs)
        self.client = manager.keypairs_client

    def list(self):
        client = self.client
        keypairs = client.list_keypairs()['keypairs']
        if not self.is_save_state:
            # recreate list removing saved keypairs
            keypairs = [keypair for keypair in keypairs
                        if keypair['keypair']['name']
                        not in self.saved_state_json['keypairs'].keys()]
        LOG.debug("List count, %s Keypairs", len(keypairs))
        return keypairs

    def delete(self):
        client = self.client
        keypairs = self.list()
        for k in keypairs:
            name = k['keypair']['name']
            try:
                LOG.debug("Deleting keypair %s", name)
                client.delete_keypair(name)
            except Exception:
                LOG.exception("Delete Keypair %s exception.", name)

    def dry_run(self):
        keypairs = self.list()
        self.data['keypairs'] = keypairs

    def save_state(self):
        keypairs = self.list()
        self.data['keypairs'] = {}
        for keypair in keypairs:
            keypair = keypair['keypair']
            self.data['keypairs'][keypair['name']] = keypair


class VolumeService(BaseService):
    def __init__(self, manager, **kwargs):
        super(VolumeService, self).__init__(kwargs)
        self.client = manager.volumes_client_latest

    def list(self):
        client = self.client
        vols = client.list_volumes()['volumes']
        if not self.is_save_state:
            # recreate list removing saved volumes
            vols = [vol for vol in vols if vol['id']
                    not in self.saved_state_json['volumes'].keys()]
        LOG.debug("List count, %s Volumes", len(vols))
        return vols

    def delete(self):
        client = self.client
        vols = self.list()
        for v in vols:
            try:
                LOG.debug("Deleting volume with id %s", v['id'])
                client.delete_volume(v['id'])
            except Exception:
                LOG.exception("Delete Volume %s exception.", v['id'])

    def dry_run(self):
        vols = self.list()
        self.data['volumes'] = vols

    def save_state(self):
        vols = self.list()
        self.data['volumes'] = {}
        for vol in vols:
            self.data['volumes'][vol['id']] = vol['name']


class VolumeQuotaService(BaseService):
    def __init__(self, manager, **kwargs):
        super(VolumeQuotaService, self).__init__(kwargs)
        self.client = manager.volume_quotas_client_latest

    def delete(self):
        client = self.client
        try:
            LOG.debug("Deleting Volume Quotas for project with id %s",
                      self.project_id)
            client.delete_quota_set(self.project_id)
        except Exception:
            LOG.exception("Delete Volume Quotas exception for 'project %s'.",
                          self.project_id)

    def dry_run(self):
        quotas = self.client.show_quota_set(
            self.project_id, params={'usage': True})['quota_set']
        self.data['volume_quotas'] = quotas


class NovaQuotaService(BaseService):
    def __init__(self, manager, **kwargs):
        super(NovaQuotaService, self).__init__(kwargs)
        self.client = manager.quotas_client
        self.limits_client = manager.limits_client

    def delete(self):
        client = self.client
        try:
            LOG.debug("Deleting Nova Quotas for project with id %s",
                      self.project_id)
            client.delete_quota_set(self.project_id)
        except Exception:
            LOG.exception("Delete Nova Quotas exception for 'project %s'.",
                          self.project_id)

    def dry_run(self):
        client = self.limits_client
        quotas = client.show_limits()['limits']
        self.data['compute_quotas'] = quotas['absolute']


class NetworkQuotaService(BaseService):
    def __init__(self, manager, **kwargs):
        super(NetworkQuotaService, self).__init__(kwargs)
        self.client = manager.network_quotas_client

    def delete(self):
        client = self.client
        try:
            LOG.debug("Deleting Network Quotas for project with id %s",
                      self.project_id)
            client.reset_quotas(self.project_id)
        except Exception:
            LOG.exception("Delete Network Quotas exception for 'project %s'.",
                          self.project_id)

    def dry_run(self):
        resp = [quota for quota in self.client.list_quotas()['quotas']
                if quota['project_id'] == self.project_id]
        self.data['network_quotas'] = resp


# Begin network service classes
class BaseNetworkService(BaseService):
    def __init__(self, manager, **kwargs):
        super(BaseNetworkService, self).__init__(kwargs)
        self.networks_client = manager.networks_client
        self.subnets_client = manager.subnets_client
        self.ports_client = manager.ports_client
        self.floating_ips_client = manager.floating_ips_client
        self.metering_labels_client = manager.metering_labels_client
        self.metering_label_rules_client = manager.metering_label_rules_client
        self.security_groups_client = manager.security_groups_client
        self.routers_client = manager.routers_client
        self.subnetpools_client = manager.subnetpools_client

    def _filter_by_conf_networks(self, item_list):
        if not item_list or not all(('network_id' in i for i in item_list)):
            return item_list

        return [item for item in item_list if item['network_id']
                not in CONF_NETWORKS]


class NetworkService(BaseNetworkService):

    def list(self):
        client = self.networks_client
        networks = client.list_networks(**self.tenant_filter)
        networks = networks['networks']

        if not self.is_save_state:
            # recreate list removing saved networks
            networks = [network for network in networks if network['id']
                        not in self.saved_state_json['networks'].keys()]
        # filter out networks declared in tempest.conf
        if self.is_preserve:
            networks = [network for network in networks
                        if network['id'] not in CONF_NETWORKS]
        LOG.debug("List count, %s Networks", len(networks))
        return networks

    def delete(self):
        client = self.networks_client
        networks = self.list()
        for n in networks:
            try:
                LOG.debug("Deleting Network with id %s", n['id'])
                client.delete_network(n['id'])
            except Exception:
                LOG.exception("Delete Network %s exception.", n['id'])

    def dry_run(self):
        networks = self.list()
        self.data['networks'] = networks

    def save_state(self):
        networks = self.list()
        self.data['networks'] = {}
        for network in networks:
            self.data['networks'][network['id']] = network


class NetworkFloatingIpService(BaseNetworkService):

    def list(self):
        client = self.floating_ips_client
        flips = client.list_floatingips(**self.tenant_filter)
        flips = flips['floatingips']

        if not self.is_save_state:
            # recreate list removing saved flips
            flips = [flip for flip in flips if flip['id']
                     not in self.saved_state_json['floatingips'].keys()]
        LOG.debug("List count, %s Network Floating IPs", len(flips))
        return flips

    def delete(self):
        client = self.floating_ips_client
        flips = self.list()
        for flip in flips:
            try:
                LOG.debug("Deleting Network Floating IP with id %s",
                          flip['id'])
                client.delete_floatingip(flip['id'])
            except Exception:
                LOG.exception("Delete Network Floating IP %s exception.",
                              flip['id'])

    def dry_run(self):
        flips = self.list()
        self.data['floatingips'] = flips

    def save_state(self):
        flips = self.list()
        self.data['floatingips'] = {}
        for flip in flips:
            self.data['floatingips'][flip['id']] = flip


class NetworkRouterService(BaseNetworkService):

    def list(self):
        client = self.routers_client
        routers = client.list_routers(**self.tenant_filter)
        routers = routers['routers']

        if not self.is_save_state:
            # recreate list removing saved routers
            routers = [router for router in routers if router['id']
                       not in self.saved_state_json['routers'].keys()]
        if self.is_preserve:
            routers = [router for router in routers
                       if router['id'] != CONF_PUB_ROUTER]

        LOG.debug("List count, %s Routers", len(routers))
        return routers

    def delete(self):
        client = self.routers_client
        ports_client = self.ports_client
        routers = self.list()
        for router in routers:
            rid = router['id']
            ports = [port for port
                     in ports_client.list_ports(device_id=rid)['ports']
                     if net_info.is_router_interface_port(port)]
            for port in ports:
                try:
                    LOG.debug("Deleting port with id %s of router with id %s",
                              port['id'], rid)
                    client.remove_router_interface(rid, port_id=port['id'])
                except Exception:
                    LOG.exception("Delete Router Interface exception for "
                                  "'port %s' of 'router %s'.", port['id'], rid)
            try:
                LOG.debug("Deleting Router with id %s", rid)
                client.delete_router(rid)
            except Exception:
                LOG.exception("Delete Router %s exception.", rid)

    def dry_run(self):
        routers = self.list()
        self.data['routers'] = routers

    def save_state(self):
        routers = self.list()
        self.data['routers'] = {}
        for router in routers:
            self.data['routers'][router['id']] = router['name']


class NetworkMeteringLabelRuleService(NetworkService):

    def list(self):
        client = self.metering_label_rules_client
        rules = client.list_metering_label_rules()
        rules = rules['metering_label_rules']
        rules = self._filter_by_tenant_id(rules)

        if not self.is_save_state:
            saved_rules = self.saved_state_json['metering_label_rules'].keys()
            # recreate list removing saved rules
            rules = [rule for rule in rules if rule['id'] not in saved_rules]
        LOG.debug("List count, %s Metering Label Rules", len(rules))
        return rules

    def delete(self):
        client = self.metering_label_rules_client
        rules = self.list()
        for rule in rules:
            try:
                LOG.debug("Deleting Metering Label Rule with id %s",
                          rule['id'])
                client.delete_metering_label_rule(rule['id'])
            except Exception:
                LOG.exception("Delete Metering Label Rule %s exception.",
                              rule['id'])

    def dry_run(self):
        rules = self.list()
        self.data['metering_label_rules'] = rules

    def save_state(self):
        rules = self.list()
        self.data['metering_label_rules'] = {}
        for rule in rules:
            self.data['metering_label_rules'][rule['id']] = rule


class NetworkMeteringLabelService(BaseNetworkService):

    def list(self):
        client = self.metering_labels_client
        labels = client.list_metering_labels()
        labels = labels['metering_labels']
        labels = self._filter_by_tenant_id(labels)

        if not self.is_save_state:
            # recreate list removing saved labels
            labels = [label for label in labels if label['id']
                      not in self.saved_state_json['metering_labels'].keys()]
        LOG.debug("List count, %s Metering Labels", len(labels))
        return labels

    def delete(self):
        client = self.metering_labels_client
        labels = self.list()
        for label in labels:
            try:
                LOG.debug("Deleting Metering Label with id %s", label['id'])
                client.delete_metering_label(label['id'])
            except Exception:
                LOG.exception("Delete Metering Label %s exception.",
                              label['id'])

    def dry_run(self):
        labels = self.list()
        self.data['metering_labels'] = labels

    def save_state(self):
        labels = self.list()
        self.data['metering_labels'] = {}
        for label in labels:
            self.data['metering_labels'][label['id']] = label['name']


class NetworkPortService(BaseNetworkService):

    def list(self):
        client = self.ports_client
        ports = [port for port in
                 client.list_ports(**self.tenant_filter)['ports']
                 if port["device_owner"] == "" or
                 port["device_owner"].startswith("compute:")]

        if not self.is_save_state:
            # recreate list removing saved ports
            ports = [port for port in ports if port['id']
                     not in self.saved_state_json['ports'].keys()]
        if self.is_preserve:
            ports = self._filter_by_conf_networks(ports)

        LOG.debug("List count, %s Ports", len(ports))
        return ports

    def delete(self):
        client = self.ports_client
        ports = self.list()
        for port in ports:
            try:
                LOG.debug("Deleting port with id %s", port['id'])
                client.delete_port(port['id'])
            except Exception:
                LOG.exception("Delete Port %s exception.", port['id'])

    def dry_run(self):
        ports = self.list()
        self.data['ports'] = ports

    def save_state(self):
        ports = self.list()
        self.data['ports'] = {}
        for port in ports:
            self.data['ports'][port['id']] = port['name']


class NetworkSecGroupService(BaseNetworkService):
    def list(self):
        client = self.security_groups_client
        filter = self.tenant_filter
        # cannot delete default sec group so never show it.
        secgroups = [secgroup for secgroup in
                     client.list_security_groups(**filter)['security_groups']
                     if secgroup['name'] != 'default']

        if not self.is_save_state:
            # recreate list removing saved security_groups
            secgroups = [secgroup for secgroup in secgroups if secgroup['id']
                         not in self.saved_state_json['security_groups'].keys()
                         ]
        if self.is_preserve:
            secgroups = [secgroup for secgroup in secgroups
                         if secgroup['security_group_rules'][0]['project_id']
                         not in CONF_PROJECTS]
        LOG.debug("List count, %s security_groups", len(secgroups))
        return secgroups

    def delete(self):
        client = self.security_groups_client
        secgroups = self.list()
        for secgroup in secgroups:
            try:
                LOG.debug("Deleting security_group with id %s", secgroup['id'])
                client.delete_security_group(secgroup['id'])
            except Exception:
                LOG.exception("Delete security_group %s exception.",
                              secgroup['id'])

    def dry_run(self):
        secgroups = self.list()
        self.data['security_groups'] = secgroups

    def save_state(self):
        secgroups = self.list()
        self.data['security_groups'] = {}
        for secgroup in secgroups:
            self.data['security_groups'][secgroup['id']] = secgroup['name']


class NetworkSubnetService(BaseNetworkService):

    def list(self):
        client = self.subnets_client
        subnets = client.list_subnets(**self.tenant_filter)
        subnets = subnets['subnets']
        if not self.is_save_state:
            # recreate list removing saved subnets
            subnets = [subnet for subnet in subnets if subnet['id']
                       not in self.saved_state_json['subnets'].keys()]
        if self.is_preserve:
            subnets = self._filter_by_conf_networks(subnets)
        LOG.debug("List count, %s Subnets", len(subnets))
        return subnets

    def delete(self):
        client = self.subnets_client
        subnets = self.list()
        for subnet in subnets:
            try:
                LOG.debug("Deleting subnet with id %s", subnet['id'])
                client.delete_subnet(subnet['id'])
            except Exception:
                LOG.exception("Delete Subnet %s exception.", subnet['id'])

    def dry_run(self):
        subnets = self.list()
        self.data['subnets'] = subnets

    def save_state(self):
        subnets = self.list()
        self.data['subnets'] = {}
        for subnet in subnets:
            self.data['subnets'][subnet['id']] = subnet['name']


class NetworkSubnetPoolsService(BaseNetworkService):

    def list(self):
        client = self.subnetpools_client
        pools = client.list_subnetpools(**self.tenant_filter)['subnetpools']
        if not self.is_save_state:
            # recreate list removing saved subnet pools
            pools = [pool for pool in pools if pool['id']
                     not in self.saved_state_json['subnetpools'].keys()]
        if self.is_preserve:
            pools = [pool for pool in pools if pool['project_id']
                     not in CONF_PROJECTS]
        LOG.debug("List count, %s Subnet Pools", len(pools))
        return pools

    def delete(self):
        client = self.subnetpools_client
        pools = self.list()
        for pool in pools:
            try:
                LOG.debug("Deleting Subnet Pool with id %s", pool['id'])
                client.delete_subnetpool(pool['id'])
            except Exception:
                LOG.exception("Delete Subnet Pool %s exception.", pool['id'])

    def dry_run(self):
        pools = self.list()
        self.data['subnetpools'] = pools

    def save_state(self):
        pools = self.list()
        self.data['subnetpools'] = {}
        for pool in pools:
            self.data['subnetpools'][pool['id']] = pool['name']


# begin global services
class RegionService(BaseService):

    def __init__(self, manager, **kwargs):
        super(RegionService, self).__init__(kwargs)
        self.client = manager.regions_client

    def list(self):
        client = self.client
        regions = client.list_regions()
        if not self.is_save_state:
            regions = [region for region in regions['regions'] if region['id']
                       not in self.saved_state_json['regions'].keys()]
            LOG.debug("List count, %s Regions", len(regions))
            return regions
        else:
            LOG.debug("List count, %s Regions", len(regions['regions']))
            return regions['regions']

    def delete(self):
        client = self.client
        regions = self.list()
        for region in regions:
            try:
                LOG.debug("Deleting region with id %s", region['id'])
                client.delete_region(region['id'])
            except Exception:
                LOG.exception("Delete Region %s exception.", region['id'])

    def dry_run(self):
        regions = self.list()
        self.data['regions'] = {}
        for region in regions:
            self.data['regions'][region['id']] = region

    def save_state(self):
        regions = self.list()
        self.data['regions'] = {}
        for region in regions:
            self.data['regions'][region['id']] = region


class FlavorService(BaseService):
    def __init__(self, manager, **kwargs):
        super(FlavorService, self).__init__(kwargs)
        self.client = manager.flavors_client

    def list(self):
        client = self.client
        flavors = client.list_flavors({"is_public": None})['flavors']
        if not self.is_save_state:
            # recreate list removing saved flavors
            flavors = [flavor for flavor in flavors if flavor['id']
                       not in self.saved_state_json['flavors'].keys()]

        if self.is_preserve:
            flavors = [flavor for flavor in flavors
                       if flavor['id'] not in CONF_FLAVORS]
        LOG.debug("List count, %s Flavors after reconcile", len(flavors))
        return flavors

    def delete(self):
        client = self.client
        flavors = self.list()
        for flavor in flavors:
            try:
                LOG.debug("Deleting flavor with id %s", flavor['id'])
                client.delete_flavor(flavor['id'])
            except Exception:
                LOG.exception("Delete Flavor %s exception.", flavor['id'])

    def dry_run(self):
        flavors = self.list()
        self.data['flavors'] = flavors

    def save_state(self):
        flavors = self.list()
        self.data['flavors'] = {}
        for flavor in flavors:
            self.data['flavors'][flavor['id']] = flavor['name']


class ImageService(BaseService):
    def __init__(self, manager, **kwargs):
        super(ImageService, self).__init__(kwargs)
        self.client = manager.image_client_v2

    def list(self):
        client = self.client
        response = client.list_images()
        images = []
        images.extend(response['images'])
        while 'next' in response:
            parsed = urllib.urlparse(response['next'])
            marker = urllib.parse_qs(parsed.query)['marker'][0]
            response = client.list_images(params={"marker": marker})
            images.extend(response['images'])

        if not self.is_save_state:
            images = [image for image in images if image['id']
                      not in self.saved_state_json['images'].keys()]
        if self.is_preserve:
            images = [image for image in images
                      if image['id'] not in CONF_IMAGES]
        LOG.debug("List count, %s Images after reconcile", len(images))
        return images

    def delete(self):
        client = self.client
        images = self.list()
        for image in images:
            try:
                LOG.debug("Deleting image with id %s", image['id'])
                client.delete_image(image['id'])
            except Exception:
                LOG.exception("Delete Image %s exception.", image['id'])

    def dry_run(self):
        images = self.list()
        self.data['images'] = images

    def save_state(self):
        self.data['images'] = {}
        images = self.list()
        for image in images:
            self.data['images'][image['id']] = image['name']


class UserService(BaseService):

    def __init__(self, manager, **kwargs):
        super(UserService, self).__init__(kwargs)
        self.client = manager.users_v3_client

    def list(self):
        users = self.client.list_users()['users']

        if not self.is_save_state:
            users = [user for user in users if user['id']
                     not in self.saved_state_json['users'].keys()]

        if self.is_preserve:
            users = [user for user in users if user['name']
                     not in CONF_USERS]

        elif not self.is_save_state:  # Never delete admin user
            users = [user for user in users if user['name'] !=
                     CONF.auth.admin_username]

        LOG.debug("List count, %s Users after reconcile", len(users))
        return users

    def delete(self):
        users = self.list()
        for user in users:
            try:
                LOG.debug("Deleting user with id %s", user['id'])
                self.client.delete_user(user['id'])
            except Exception:
                LOG.exception("Delete User %s exception.", user['id'])

    def dry_run(self):
        users = self.list()
        self.data['users'] = users

    def save_state(self):
        users = self.list()
        self.data['users'] = {}
        for user in users:
            self.data['users'][user['id']] = user['name']


class RoleService(BaseService):

    def __init__(self, manager, **kwargs):
        super(RoleService, self).__init__(kwargs)
        self.client = manager.roles_v3_client

    def list(self):
        try:
            roles = self.client.list_roles()['roles']
            # reconcile roles with saved state and never list admin role
            if not self.is_save_state:
                roles = [role for role in roles if
                         (role['id'] not in
                          self.saved_state_json['roles'].keys() and
                          role['name'] != CONF.identity.admin_role)]
                LOG.debug("List count, %s Roles after reconcile", len(roles))
            return roles
        except Exception:
            LOG.exception("Cannot retrieve Roles.")
            return []

    def delete(self):
        roles = self.list()
        for role in roles:
            try:
                LOG.debug("Deleting role with id %s", role['id'])
                self.client.delete_role(role['id'])
            except Exception:
                LOG.exception("Delete Role %s exception.", role['id'])

    def dry_run(self):
        roles = self.list()
        self.data['roles'] = roles

    def save_state(self):
        roles = self.list()
        self.data['roles'] = {}
        for role in roles:
            self.data['roles'][role['id']] = role['name']


class ProjectService(BaseService):

    def __init__(self, manager, **kwargs):
        super(ProjectService, self).__init__(kwargs)
        self.client = manager.projects_client

    def list(self):
        projects = self.client.list_projects()['projects']
        if not self.is_save_state:
            project_ids = self.saved_state_json['projects']
            projects = [project
                        for project in projects
                        if (project['id'] not in project_ids and
                            project['name'] != CONF.auth.admin_project_name)]

        if self.is_preserve:
            projects = [project
                        for project in projects
                        if project['name'] not in CONF_PROJECTS]

        LOG.debug("List count, %s Projects after reconcile", len(projects))
        return projects

    def delete(self):
        projects = self.list()
        for project in projects:
            try:
                LOG.debug("Deleting project with id %s", project['id'])
                self.client.delete_project(project['id'])
            except Exception:
                LOG.exception("Delete project %s exception.", project['id'])

    def dry_run(self):
        projects = self.list()
        self.data['projects'] = projects

    def save_state(self):
        projects = self.list()
        self.data['projects'] = {}
        for project in projects:
            self.data['projects'][project['id']] = project['name']


class DomainService(BaseService):

    def __init__(self, manager, **kwargs):
        super(DomainService, self).__init__(kwargs)
        self.client = manager.domains_client

    def list(self):
        client = self.client
        domains = client.list_domains()['domains']
        if not self.is_save_state:
            domains = [domain for domain in domains if domain['id']
                       not in self.saved_state_json['domains'].keys()]

        LOG.debug("List count, %s Domains after reconcile", len(domains))
        return domains

    def delete(self):
        client = self.client
        domains = self.list()
        for domain in domains:
            try:
                LOG.debug("Deleting domain with id %s", domain['id'])
                client.update_domain(domain['id'], enabled=False)
                client.delete_domain(domain['id'])
            except Exception:
                LOG.exception("Delete Domain %s exception.", domain['id'])

    def dry_run(self):
        domains = self.list()
        self.data['domains'] = domains

    def save_state(self):
        domains = self.list()
        self.data['domains'] = {}
        for domain in domains:
            self.data['domains'][domain['id']] = domain['name']


def get_project_associated_cleanup_services():
    """Returns list of project service classes.

    The list contains services whose resources need to be deleted prior,
    the project they are associated with, deletion. The resources cannot be
    most likely deleted after the project is deleted first.
    """
    project_associated_services = []
    # TODO(gmann): Tempest should provide some plugin hook for cleanup
    # script extension to plugin tests also.
    if IS_NOVA:
        project_associated_services.append(NovaQuotaService)
    if IS_CINDER:
        project_associated_services.append(VolumeQuotaService)
    if IS_NEUTRON:
        project_associated_services.append(NetworkQuotaService)
    return project_associated_services


def get_resource_cleanup_services():
    """Returns list of project related classes.

    The list contains services whose resources are associated with a project,
    however, their deletion is possible also after the project is deleted
    first.
    """
    resource_cleanup_services = []
    # TODO(gmann): Tempest should provide some plugin hook for cleanup
    # script extension to plugin tests also.
    if IS_NOVA:
        resource_cleanup_services.append(ServerService)
        resource_cleanup_services.append(KeyPairService)
        resource_cleanup_services.append(ServerGroupService)
    if IS_NEUTRON:
        resource_cleanup_services.append(NetworkFloatingIpService)
        if utils.is_extension_enabled('metering', 'network'):
            resource_cleanup_services.append(NetworkMeteringLabelRuleService)
            resource_cleanup_services.append(NetworkMeteringLabelService)
        resource_cleanup_services.append(NetworkRouterService)
        resource_cleanup_services.append(NetworkPortService)
        resource_cleanup_services.append(NetworkSubnetService)
        resource_cleanup_services.append(NetworkService)
        resource_cleanup_services.append(NetworkSecGroupService)
        resource_cleanup_services.append(NetworkSubnetPoolsService)
    if IS_CINDER:
        resource_cleanup_services.append(SnapshotService)
        resource_cleanup_services.append(VolumeService)
    return resource_cleanup_services


def get_global_cleanup_services():
    global_services = []
    if IS_NOVA:
        global_services.append(FlavorService)
    if IS_GLANCE:
        global_services.append(ImageService)
    global_services.append(UserService)
    global_services.append(ProjectService)
    global_services.append(DomainService)
    global_services.append(RoleService)
    global_services.append(RegionService)
    return global_services



# Copyright 2013 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.compute import base
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc


class ImagesMetadataNegativeTestJSON(base.BaseV2ComputeTest):
    """Negative tests of image metadata

    Negative tests of image metadata with compute microversion less than 2.39.
    """

    max_microversion = '2.38'

    @classmethod
    def setup_clients(cls):
        super(ImagesMetadataNegativeTestJSON, cls).setup_clients()
        cls.client = cls.compute_images_client

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('94069db2-792f-4fa8-8bd3-2271a6e0c095')
    def test_list_nonexistent_image_metadata(self):
        """Test listing metadata of a non existence image should fail"""
        self.assertRaises(lib_exc.NotFound, self.client.list_image_metadata,
                          data_utils.rand_uuid())

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('a403ef9e-9f95-427c-b70a-3ce3388796f1')
    def test_update_nonexistent_image_metadata(self):
        """Test updating metadata of a non existence image should fail"""
        meta = {'os_distro': 'alt1', 'os_version': 'alt2'}
        self.assertRaises(lib_exc.NotFound,
                          self.client.update_image_metadata,
                          data_utils.rand_uuid(), meta)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('41ae052c-6ee6-405c-985e-5712393a620d')
    def test_get_nonexistent_image_metadata_item(self):
        """Test getting metadata of a non existence image should fail"""
        self.assertRaises(lib_exc.NotFound,
                          self.client.show_image_metadata_item,
                          data_utils.rand_uuid(), 'os_version')

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('dc64f2ce-77e8-45b0-88c8-e15041d08eaf')
    def test_set_nonexistent_image_metadata(self):
        """Test setting metadata of a non existence image should fail"""
        meta = {'os_distro': 'alt1', 'os_version': 'alt2'}
        self.assertRaises(lib_exc.NotFound, self.client.set_image_metadata,
                          data_utils.rand_uuid(), meta)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('2154fd03-ab54-457c-8874-e6e3eb56e9cf')
    def test_set_nonexistent_image_metadata_item(self):
        """Test setting metadata item of a non existence image should fail"""
        meta = {'os_distro': 'alt'}
        self.assertRaises(lib_exc.NotFound,
                          self.client.set_image_metadata_item,
                          data_utils.rand_uuid(), 'os_distro',
                          meta)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('848e157f-6bcf-4b2e-a5dd-5124025a8518')
    def test_delete_nonexistent_image_metadata_item(self):
        """Test deleting metadata item of a non existence image should fail"""
        self.assertRaises(lib_exc.NotFound,
                          self.client.delete_image_metadata_item,
                          data_utils.rand_uuid(), 'os_distro')



# Copyright 2013 IBM Corporation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


from tempest.api.compute import base
from tempest.lib import decorators


class ServerPasswordTestJSON(base.BaseV2ComputeTest):
    """Test server password"""

    create_default_network = True

    @classmethod
    def resource_setup(cls):
        super(ServerPasswordTestJSON, cls).resource_setup()
        cls.server = cls.create_test_server(wait_until="ACTIVE")

    @decorators.idempotent_id('f83b582f-62a8-4f22-85b0-0dee50ff783a')
    def test_get_server_password(self):
        """Test getting password of a server"""
        self.servers_client.show_password(self.server['id'])

    @decorators.idempotent_id('f8229e8b-b625-4493-800a-bde86ac611ea')
    def test_delete_server_password(self):
        """Test deleting password from a server"""
        self.servers_client.delete_password(self.server['id'])



# Copyright 2014 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

common_instance_usage_audit_log = {
    'type': 'object',
    'properties': {
        'hosts_not_run': {
            'type': 'array',
            'items': {'type': 'string'}
        },
        'log': {
            'type': 'object',
            'patternProperties': {
                # NOTE: Here is a host name.
                '^.+$': {
                    'type': 'object',
                    'properties': {
                        'state': {'type': 'string'},
                        'instances': {'type': 'integer'},
                        'errors': {'type': 'integer'},
                        'message': {'type': 'string'}
                    },
                    'additionalProperties': False,
                    'required': ['state', 'instances', 'errors', 'message']
                }
            }
        },
        'num_hosts': {'type': 'integer'},
        'num_hosts_done': {'type': 'integer'},
        'num_hosts_not_run': {'type': 'integer'},
        'num_hosts_running': {'type': 'integer'},
        'overall_status': {'type': 'string'},
        'period_beginning': {'type': 'string'},
        'period_ending': {'type': 'string'},
        'total_errors': {'type': 'integer'},
        'total_instances': {'type': 'integer'}
    },
    'additionalProperties': False,
    'required': ['hosts_not_run', 'log', 'num_hosts', 'num_hosts_done',
                 'num_hosts_not_run', 'num_hosts_running', 'overall_status',
                 'period_beginning', 'period_ending', 'total_errors',
                 'total_instances']
}

get_instance_usage_audit_log = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'instance_usage_audit_log': common_instance_usage_audit_log
        },
        'additionalProperties': False,
        'required': ['instance_usage_audit_log']
    }
}

list_instance_usage_audit_log = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'instance_usage_audit_logs': common_instance_usage_audit_log
        },
        'additionalProperties': False,
        'required': ['instance_usage_audit_logs']
    }
}



# Copyright 2014 Hewlett-Packard Development Company, L.P.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.common import custom_matchers
from tempest.tests import base


# Stolen from testtools/testtools/tests/matchers/helpers.py
class TestMatchersInterface(object):

    def test_matches_match(self):
        matcher = self.matches_matcher
        matches = self.matches_matches
        mismatches = self.matches_mismatches
        for candidate in matches:
            self.assertIsNone(matcher.match(candidate))
        for candidate in mismatches:
            mismatch = matcher.match(candidate)
            self.assertIsNotNone(mismatch)
            self.assertIsNotNone(getattr(mismatch, 'describe', None))

    def test__str__(self):
        # [(expected, object to __str__)].
        from testtools.matchers._doctest import DocTestMatches
        examples = self.str_examples
        for expected, matcher in examples:
            self.assertThat(matcher, DocTestMatches(expected))

    def test_describe_difference(self):
        # [(expected, matchee, matcher), ...]
        examples = self.describe_examples
        for difference, matchee, matcher in examples:
            mismatch = matcher.match(matchee)
            self.assertEqual(difference, mismatch.describe())

    def test_mismatch_details(self):
        # The mismatch object must provide get_details, which must return a
        # dictionary mapping names to Content objects.
        examples = self.describe_examples
        for difference, matchee, matcher in examples:
            mismatch = matcher.match(matchee)
            details = mismatch.get_details()
            self.assertEqual(dict(details), details)


class TestMatchesDictExceptForKeys(base.TestCase,
                                   TestMatchersInterface):

    matches_matcher = custom_matchers.MatchesDictExceptForKeys(
        {'a': 1, 'b': 2, 'c': 3, 'd': 4}, ['c', 'd'])
    matches_matches = [
        {'a': 1, 'b': 2, 'c': 3, 'd': 4},
        {'a': 1, 'b': 2, 'c': 5},
        {'a': 1, 'b': 2},
    ]
    matches_mismatches = [
        {},
        {'foo': 1},
        {'a': 1, 'b': 3},
        {'a': 1, 'b': 2, 'foo': 1},
        {'a': 1, 'b': None, 'foo': 1},
    ]

    str_examples = []
    describe_examples = [
        ("Only in expected:\n"
         "  {'a': 1, 'b': 2}\n",
         {},
         matches_matcher),
        ("Only in expected:\n"
         "  {'a': 1, 'b': 2}\n"
         "Only in actual:\n"
         "  {'foo': 1}\n",
         {'foo': 1},
         matches_matcher),
        ("Differences:\n"
         "  b: expected 2, actual 3\n",
         {'a': 1, 'b': 3},
         matches_matcher),
        ("Only in actual:\n"
         "  {'foo': 1}\n",
         {'a': 1, 'b': 2, 'foo': 1},
         matches_matcher),
        ("Only in actual:\n"
         "  {'foo': 1}\n"
         "Differences:\n"
         "  b: expected 2, actual None\n",
         {'a': 1, 'b': None, 'foo': 1},
         matches_matcher)
    ]



# Copyright 2015 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import time

import testtools

from tempest.api.identity import base
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions


CONF = config.CONF


class IdentityUsersTest(base.BaseIdentityV2Test):
    """Test user password in identity v2 API"""

    @classmethod
    def resource_setup(cls):
        super(IdentityUsersTest, cls).resource_setup()
        cls.creds = cls.os_primary.credentials
        cls.username = cls.creds.username
        cls.password = cls.creds.password
        cls.tenant_name = cls.creds.tenant_name

    def _update_password(self, user_id, original_password, password):
        self.non_admin_users_client.update_user_own_password(
            user_id, password=password, original_password=original_password)

        # NOTE(morganfainberg): Fernet tokens are not subsecond aware and
        # Keystone should only be precise to the second. Sleep to ensure
        # we are passing the second boundary.
        time.sleep(1)

        # check authorization with new password
        self.non_admin_token_client.auth(self.username,
                                         password,
                                         self.tenant_name)

        # Reset auth to get a new token with the new password
        self.non_admin_users_client.auth_provider.clear_auth()
        self.non_admin_users_client.auth_provider.credentials.password = (
            password)

    def _restore_password(self, user_id, old_pass, new_pass):
        if CONF.identity_feature_enabled.security_compliance:
            # First we need to clear the password history
            unique_count = CONF.identity.user_unique_last_password_count
            for _ in range(unique_count):
                random_pass = data_utils.rand_password()
                self._update_password(
                    user_id, original_password=new_pass, password=random_pass)
                new_pass = random_pass

        self._update_password(
            user_id, original_password=new_pass, password=old_pass)
        # Reset auth again to verify the password restore does work.
        # Clear auth restores the original credentials and deletes
        # cached auth data
        self.non_admin_users_client.auth_provider.clear_auth()
        # NOTE(lbragstad): Fernet tokens are not subsecond aware and
        # Keystone should only be precise to the second. Sleep to ensure we
        # are passing the second boundary before attempting to
        # authenticate.
        time.sleep(1)
        self.non_admin_users_client.auth_provider.set_auth()

    @decorators.idempotent_id('165859c9-277f-4124-9479-a7d1627b0ca7')
    @testtools.skipIf(CONF.identity_feature_enabled.immutable_user_source,
                      'Skipped because environment has an '
                      'immutable user source and solely '
                      'provides read-only access to users.')
    def test_user_update_own_password(self):
        """test updating user's own password via v2 API"""
        old_pass = self.creds.password
        old_token = self.non_admin_users_client.token
        new_pass = data_utils.rand_password()
        user_id = self.creds.user_id

        # to change password back. important for use_dynamic_credentials=false
        self.addCleanup(self._restore_password, user_id, old_pass, new_pass)

        # user updates own password
        self._update_password(
            user_id, original_password=old_pass, password=new_pass)

        # authorize with old token should lead to Unauthorized
        self.assertRaises(exceptions.Unauthorized,
                          self.non_admin_token_client.auth_token,
                          old_token)

        # authorize with old password should lead to Unauthorized
        self.assertRaises(exceptions.Unauthorized,
                          self.non_admin_token_client.auth,
                          self.username,
                          old_pass,
                          self.tenant_name)



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from urllib import parse as urllib

from oslo_serialization import jsonutils as json

from tempest.lib.api_schema.response.compute.v2_1 import keypairs as schemav21
from tempest.lib.api_schema.response.compute.v2_2 import keypairs as schemav22
from tempest.lib.common import rest_client
from tempest.lib.services.compute import base_compute_client


class KeyPairsClient(base_compute_client.BaseComputeClient):

    schema_versions_info = [{'min': None, 'max': '2.1', 'schema': schemav21},
                            {'min': '2.2', 'max': None, 'schema': schemav22}]

    def list_keypairs(self, **params):
        """Lists keypairs that are associated with the account.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#list-keypairs
        """
        url = 'os-keypairs'
        if params:
            url += '?%s' % urllib.urlencode(params)
        resp, body = self.get(url)
        body = json.loads(body)
        schema = self.get_schema(self.schema_versions_info)
        self.validate_response(schema.list_keypairs, resp, body)
        return rest_client.ResponseBody(resp, body)

    def show_keypair(self, keypair_name, **params):
        """Shows details for a keypair that is associated with the account.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#show-keypair-details
        """
        url = "os-keypairs/%s" % keypair_name
        if params:
            url += '?%s' % urllib.urlencode(params)
        resp, body = self.get(url)
        body = json.loads(body)
        schema = self.get_schema(self.schema_versions_info)
        self.validate_response(schema.get_keypair, resp, body)
        return rest_client.ResponseBody(resp, body)

    def create_keypair(self, **kwargs):
        """Create a keypair.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#create-or-import-keypair
        """
        post_body = json.dumps({'keypair': kwargs})
        resp, body = self.post("os-keypairs", body=post_body)
        body = json.loads(body)
        schema = self.get_schema(self.schema_versions_info)
        self.validate_response(schema.create_keypair, resp, body)
        return rest_client.ResponseBody(resp, body)

    def delete_keypair(self, keypair_name, **params):
        """Deletes a keypair.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#delete-keypair
        """
        url = "os-keypairs/%s" % keypair_name
        if params:
            url += '?%s' % urllib.urlencode(params)
        resp, body = self.delete(url)
        schema = self.get_schema(self.schema_versions_info)
        self.validate_response(schema.delete_keypair, resp, body)
        return rest_client.ResponseBody(resp, body)



# Copyright 2015 IBM Corp.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import fixtures

from tempest.lib.services.compute import base_compute_client
from tempest.lib.services.compute import server_groups_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib import fake_http
from tempest.tests.lib.services import base


class TestServerGroupsClient(base.BaseServiceTest):

    server_group = {
        "id": "5bbcc3c4-1da2-4437-a48a-66f15b1b13f9",
        "name": "test",
        "policies": ["anti-affinity"],
        "members": [],
        "metadata": {}}

    def setUp(self):
        super(TestServerGroupsClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.client = server_groups_client.ServerGroupsClient(
            fake_auth, 'compute', 'regionOne')

    def _test_create_server_group(self, bytes_body=False):
        expected = {"server_group": self.server_group}
        self.check_service_client_function(
            self.client.create_server_group,
            'tempest.lib.common.rest_client.RestClient.post', expected,
            bytes_body, name='fake-group', policies=['affinity'])

    def test_create_server_group_str_body(self):
        self._test_create_server_group(bytes_body=False)

    def test_create_server_group_byte_body(self):
        self._test_create_server_group(bytes_body=True)

    def test_delete_server_group(self):
        response = fake_http.fake_http_response({}, status=204), ''
        self.useFixture(fixtures.MockPatch(
            'tempest.lib.common.rest_client.RestClient.delete',
            return_value=response))
        self.client.delete_server_group('fake-group')

    def _test_list_server_groups(self, bytes_body=False):
        expected = {"server_groups": [self.server_group]}
        self.check_service_client_function(
            self.client.list_server_groups,
            'tempest.lib.common.rest_client.RestClient.get',
            expected, bytes_body)

    def test_list_server_groups_str_body(self):
        self._test_list_server_groups(bytes_body=False)

    def test_list_server_groups_byte_body(self):
        self._test_list_server_groups(bytes_body=True)

    def _test_show_server_group(self, bytes_body=False):
        expected = {"server_group": self.server_group}
        self.check_service_client_function(
            self.client.show_server_group,
            'tempest.lib.common.rest_client.RestClient.get',
            expected, bytes_body,
            server_group_id='5bbcc3c4-1da2-4437-a48a-66f15b1b13f9')

    def test_show_server_group_str_body(self):
        self._test_show_server_group(bytes_body=False)

    def test_show_server_group_byte_body(self):
        self._test_show_server_group(bytes_body=True)


class TestServerGroupsClientMinV213(TestServerGroupsClient):

    server_group = {
        "id": "5bbcc3c4-1da2-4437-a48a-66f15b1b13f9",
        "name": "test",
        "policies": ["anti-affinity"],
        "members": [],
        "metadata": {},
        "project_id": "0beb4bffb7a445eb8eb05fee3ee7660a",
        "user_id": "86031628064a4f99bb66ec03c507dcd8"}

    def setUp(self):
        super(TestServerGroupsClientMinV213, self).setUp()
        self.patchobject(base_compute_client, 'COMPUTE_MICROVERSION',
                         new='2.13')



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy

from tempest.lib.api_schema.response.compute.v2_26 import servers as servers226

flavor = {
    'type': 'object',
    'properties': {
        'original_name': {'type': 'string'},
        'disk': {'type': 'integer'},
        'ephemeral': {'type': 'integer'},
        'ram': {'type': 'integer'},
        'swap': {'type': 'integer'},
        'vcpus': {'type': 'integer'},
        'extra_specs': {
            'type': 'object',
            'patternProperties': {
                r'^[a-zA-Z0-9_\-\. :]+$': {'type': 'string'}
            }
        }
    },
    'additionalProperties': False,
    'required': ['original_name', 'disk', 'ephemeral', 'ram', 'swap', 'vcpus']
}

get_server = copy.deepcopy(servers226.get_server)
get_server['response_body']['properties']['server'][
    'properties'].update({'flavor': flavor})
list_servers_detail = copy.deepcopy(servers226.list_servers_detail)
list_servers_detail['response_body']['properties']['servers']['items'][
    'properties'].update({'flavor': flavor})

update_server = copy.deepcopy(servers226.update_server)
update_server['response_body']['properties']['server'][
    'properties'].update({'flavor': flavor})

rebuild_server = copy.deepcopy(servers226.rebuild_server)
rebuild_server['response_body']['properties']['server'][
    'properties'].update({'flavor': flavor})

rebuild_server_with_admin_pass = copy.deepcopy(
    servers226.rebuild_server_with_admin_pass)
rebuild_server_with_admin_pass['response_body']['properties']['server'][
    'properties'].update({'flavor': flavor})

# NOTE(zhufl): Below are the unchanged schema in this microversion. We need
# to keep this schema in this file to have the generic way to select the
# right schema based on self.schema_versions_info mapping in service client.
show_server_diagnostics = copy.deepcopy(servers226.show_server_diagnostics)
get_remote_consoles = copy.deepcopy(servers226.get_remote_consoles)
list_tags = copy.deepcopy(servers226.list_tags)
update_all_tags = copy.deepcopy(servers226.update_all_tags)
delete_all_tags = copy.deepcopy(servers226.delete_all_tags)
check_tag_existence = copy.deepcopy(servers226.check_tag_existence)
update_tag = copy.deepcopy(servers226.update_tag)
delete_tag = copy.deepcopy(servers226.delete_tag)
list_servers = copy.deepcopy(servers226.list_servers)
attach_volume = copy.deepcopy(servers226.attach_volume)
show_volume_attachment = copy.deepcopy(servers226.show_volume_attachment)
list_volume_attachments = copy.deepcopy(servers226.list_volume_attachments)



# Copyright 2013 Huawei Technologies Co.,LTD.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.identity import base
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc


class TenantsNegativeTestJSON(base.BaseIdentityV2AdminTest):
    """Negative tests of keystone tenants via v2 API"""

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('ca9bb202-63dd-4240-8a07-8ef9c19c04bb')
    def test_list_tenants_by_unauthorized_user(self):
        """Test Non-admin should not be able to list tenants via v2 API"""
        self.assertRaises(lib_exc.Forbidden,
                          self.non_admin_tenants_client.list_tenants)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('df33926c-1c96-4d8d-a762-79cc6b0c3cf4')
    def test_list_tenant_request_without_token(self):
        """Test listing tenants without a valid token via v2 API

        Listing tenants without a valid token via v2 API should fail.
        """
        token = self.client.auth_provider.get_token()
        self.client.delete_token(token)
        self.assertRaises(lib_exc.Unauthorized,
                          self.tenants_client.list_tenants)
        self.client.auth_provider.clear_auth()

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('162ba316-f18b-4987-8c0c-fd9140cd63ed')
    def test_tenant_delete_by_unauthorized_user(self):
        """Test non-admin should not be able to delete a tenant via v2 API"""
        tenant = self.setup_test_tenant()
        self.assertRaises(lib_exc.Forbidden,
                          self.non_admin_tenants_client.delete_tenant,
                          tenant['id'])

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('e450db62-2e9d-418f-893a-54772d6386b1')
    def test_tenant_delete_request_without_token(self):
        """Test deleting a tenant without a valid token via v2 API

        Deleting a tenant without a valid token via v2 API should fail.
        """
        tenant = self.setup_test_tenant()
        token = self.client.auth_provider.get_token()
        self.client.delete_token(token)
        self.assertRaises(lib_exc.Unauthorized,
                          self.tenants_client.delete_tenant,
                          tenant['id'])
        self.client.auth_provider.clear_auth()

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('9c9a2aed-6e3c-467a-8f5c-89da9d1b516b')
    def test_delete_non_existent_tenant(self):
        """Test deleting a non existent tenant via v2 API should fail"""
        self.assertRaises(lib_exc.NotFound, self.tenants_client.delete_tenant,
                          data_utils.rand_uuid_hex())

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('af16f44b-a849-46cb-9f13-a751c388f739')
    def test_tenant_create_duplicate(self):
        """Test tenant names should be unique via v2 API"""
        tenant_name = data_utils.rand_name(name='tenant')
        self.setup_test_tenant(name=tenant_name)
        self.assertRaises(lib_exc.Conflict, self.tenants_client.create_tenant,
                          name=tenant_name)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('d26b278a-6389-4702-8d6e-5980d80137e0')
    def test_create_tenant_by_unauthorized_user(self):
        """Test non-admin user creating a tenant via v2 API

        Non-admin user should not be authorized to create a tenant via v2 API.
        """
        tenant_name = data_utils.rand_name(name='tenant')
        self.assertRaises(lib_exc.Forbidden,
                          self.non_admin_tenants_client.create_tenant,
                          name=tenant_name)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('a3ee9d7e-6920-4dd5-9321-d4b2b7f0a638')
    def test_create_tenant_request_without_token(self):
        """Test creating tenant without a token via v2 API is not allowed"""
        tenant_name = data_utils.rand_name(name='tenant')
        token = self.client.auth_provider.get_token()
        self.client.delete_token(token)
        self.assertRaises(lib_exc.Unauthorized,
                          self.tenants_client.create_tenant,
                          name=tenant_name)
        self.client.auth_provider.clear_auth()

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('5a2e4ca9-b0c0-486c-9c48-64a94fba2395')
    def test_create_tenant_with_empty_name(self):
        """Test tenant name should not be empty via v2 API"""
        self.assertRaises(lib_exc.BadRequest,
                          self.tenants_client.create_tenant,
                          name='')

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('2ff18d1e-dfe3-4359-9dc3-abf582c196b9')
    def test_create_tenants_name_length_over_64(self):
        """Test tenant name length should not exceed 64 via v2 API"""
        tenant_name = 'a' * 65
        self.assertRaises(lib_exc.BadRequest,
                          self.tenants_client.create_tenant,
                          name=tenant_name)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('bd20dc2a-9557-4db7-b755-f48d952ad706')
    def test_update_non_existent_tenant(self):
        """Test updating a non existent tenant via v2 API should fail"""
        self.assertRaises(lib_exc.NotFound, self.tenants_client.update_tenant,
                          data_utils.rand_uuid_hex())

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('41704dc5-c5f7-4f79-abfa-76e6fedc570b')
    def test_tenant_update_by_unauthorized_user(self):
        """Test non-admin user updating a tenant via v2 API

        Non-admin user should not be able to update a tenant via v2 API
        """
        tenant = self.setup_test_tenant()
        self.assertRaises(lib_exc.Forbidden,
                          self.non_admin_tenants_client.update_tenant,
                          tenant['id'])

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('7a421573-72c7-4c22-a98e-ce539219c657')
    def test_tenant_update_request_without_token(self):
        """Test updating a tenant without a valid token via v2 API

        Updating a tenant without a valid token via v2 API should fail
        """
        tenant = self.setup_test_tenant()
        token = self.client.auth_provider.get_token()
        self.client.delete_token(token)
        self.assertRaises(lib_exc.Unauthorized,
                          self.tenants_client.update_tenant,
                          tenant['id'])
        self.client.auth_provider.clear_auth()



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.identity import base
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators


class TenantsTestJSON(base.BaseIdentityV2AdminTest):
    """Test identity tenants via v2 API"""

    @decorators.idempotent_id('16c6e05c-6112-4b0e-b83f-5e43f221b6b0')
    def test_tenant_list_delete(self):
        """Test listing and deleting tenants via v2 API

        Create several tenants and delete them
        """
        tenants = []
        for _ in range(3):
            tenant = self.setup_test_tenant()
            tenants.append(tenant)
        tenant_ids = [tn['id'] for tn in tenants]
        body = self.tenants_client.list_tenants()['tenants']
        found = [t for t in body if t['id'] in tenant_ids]
        self.assertEqual(len(found), len(tenants), 'Tenants not created')

        for tenant in tenants:
            self.tenants_client.delete_tenant(tenant['id'])

        body = self.tenants_client.list_tenants()['tenants']
        found = [tenant for tenant in body if tenant['id'] in tenant_ids]
        self.assertEmpty(found, 'Tenants failed to delete')

    @decorators.idempotent_id('d25e9f24-1310-4d29-b61b-d91299c21d6d')
    def test_tenant_create_with_description(self):
        """Test creating tenant with a description via v2 API"""
        tenant_desc = data_utils.rand_name(name='desc')
        tenant = self.setup_test_tenant(description=tenant_desc)
        tenant_id = tenant['id']
        desc1 = tenant['description']
        self.assertEqual(desc1, tenant_desc, 'Description should have '
                         'been sent in response for create')
        body = self.tenants_client.show_tenant(tenant_id)['tenant']
        desc2 = body['description']
        self.assertEqual(desc2, tenant_desc, 'Description does not appear '
                         'to be set')
        self.tenants_client.delete_tenant(tenant_id)

    @decorators.idempotent_id('670bdddc-1cd7-41c7-b8e2-751cfb67df50')
    def test_tenant_create_enabled(self):
        """Test creating a tenant that is enabled via v2 API"""
        tenant = self.setup_test_tenant(enabled=True)
        tenant_id = tenant['id']
        self.assertTrue(tenant['enabled'], 'Enable should be True in response')
        body = self.tenants_client.show_tenant(tenant_id)['tenant']
        self.assertTrue(body['enabled'], 'Enable should be True in lookup')
        self.tenants_client.delete_tenant(tenant_id)

    @decorators.idempotent_id('3be22093-b30f-499d-b772-38340e5e16fb')
    def test_tenant_create_not_enabled(self):
        """Test creating a tenant that is not enabled via v2 API"""
        tenant = self.setup_test_tenant(enabled=False)
        tenant_id = tenant['id']
        self.assertFalse(tenant['enabled'],
                         'Enable should be False in response')
        body = self.tenants_client.show_tenant(tenant_id)['tenant']
        self.assertFalse(body['enabled'],
                         'Enable should be False in lookup')
        self.tenants_client.delete_tenant(tenant_id)

    @decorators.idempotent_id('781f2266-d128-47f3-8bdb-f70970add238')
    def test_tenant_update_name(self):
        """Test updating name attribute of a tenant via v2 API"""
        t_name1 = data_utils.rand_name(name='tenant')
        tenant = self.setup_test_tenant(name=t_name1)
        t_id = tenant['id']
        resp1_name = tenant['name']

        t_name2 = data_utils.rand_name(name='tenant2')
        body = self.tenants_client.update_tenant(t_id, name=t_name2)['tenant']
        resp2_name = body['name']
        self.assertNotEqual(resp1_name, resp2_name)

        body = self.tenants_client.show_tenant(t_id)['tenant']
        resp3_name = body['name']

        self.assertNotEqual(resp1_name, resp3_name)
        self.assertEqual(t_name1, resp1_name)
        self.assertEqual(resp2_name, resp3_name)

        self.tenants_client.delete_tenant(t_id)

    @decorators.idempotent_id('859fcfe1-3a03-41ef-86f9-b19a47d1cd87')
    def test_tenant_update_desc(self):
        """Test updating description attribute of a tenant via v2 API"""
        t_desc = data_utils.rand_name(name='desc')
        tenant = self.setup_test_tenant(description=t_desc)
        t_id = tenant['id']
        resp1_desc = tenant['description']

        t_desc2 = data_utils.rand_name(name='desc2')
        body = self.tenants_client.update_tenant(t_id, description=t_desc2)
        updated_tenant = body['tenant']
        resp2_desc = updated_tenant['description']
        self.assertNotEqual(resp1_desc, resp2_desc)

        body = self.tenants_client.show_tenant(t_id)['tenant']
        resp3_desc = body['description']

        self.assertNotEqual(resp1_desc, resp3_desc)
        self.assertEqual(t_desc, resp1_desc)
        self.assertEqual(resp2_desc, resp3_desc)

        self.tenants_client.delete_tenant(t_id)

    @decorators.idempotent_id('8fc8981f-f12d-4c66-9972-2bdcf2bc2e1a')
    def test_tenant_update_enable(self):
        """Test updating the enabled attribute of a tenant via v2 API"""
        t_en = False
        tenant = self.setup_test_tenant(enabled=t_en)
        t_id = tenant['id']
        resp1_en = tenant['enabled']

        t_en2 = True
        body = self.tenants_client.update_tenant(t_id, enabled=t_en2)
        updated_tenant = body['tenant']
        resp2_en = updated_tenant['enabled']
        self.assertNotEqual(resp1_en, resp2_en)

        body = self.tenants_client.show_tenant(t_id)['tenant']
        resp3_en = body['enabled']

        self.assertNotEqual(resp1_en, resp3_en)
        self.assertFalse(tenant['enabled'])
        self.assertEqual(resp2_en, resp3_en)

        self.tenants_client.delete_tenant(t_id)



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.volume import base
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc


class ExtraSpecsNegativeTest(base.BaseVolumeAdminTest):
    """Negative tests of volume type extra specs"""

    @classmethod
    def resource_setup(cls):
        super(ExtraSpecsNegativeTest, cls).resource_setup()
        extra_specs = {"spec1": "val1"}
        cls.volume_type = cls.create_volume_type(extra_specs=extra_specs)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('08961d20-5cbb-4910-ac0f-89ad6dbb2da1')
    def test_update_no_body(self):
        """Test updating volume type extra specs with no body should fail"""
        self.assertRaises(
            lib_exc.BadRequest,
            self.admin_volume_types_client.update_volume_type_extra_specs,
            self.volume_type['id'], "spec1", None)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('25e5a0ee-89b3-4c53-8310-236f76c75365')
    def test_update_nonexistent_extra_spec_id(self):
        """Test updating volume type extra specs with non existent name

        Updating volume type extra specs with non existent extra spec name
        should fail.
        """
        extra_spec = {"spec1": "val2"}
        self.assertRaises(
            lib_exc.BadRequest,
            self.admin_volume_types_client.update_volume_type_extra_specs,
            self.volume_type['id'], data_utils.rand_uuid(),
            extra_spec)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('9bf7a657-b011-4aec-866d-81c496fbe5c8')
    def test_update_none_extra_spec_id(self):
        """Test updating volume type extra specs without name

        Updating volume type extra specs without extra spec name should fail.
        """
        extra_spec = {"spec1": "val2"}
        self.assertRaises(
            lib_exc.BadRequest,
            self.admin_volume_types_client.update_volume_type_extra_specs,
            self.volume_type['id'], None, extra_spec)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('a77dfda2-9100-448e-9076-ed1711f4bdfc')
    def test_update_multiple_extra_spec(self):
        """Test updating multiple volume type extra specs should fail"""
        extra_spec = {"spec1": "val2", "spec2": "val1"}
        self.assertRaises(
            lib_exc.BadRequest,
            self.admin_volume_types_client.update_volume_type_extra_specs,
            self.volume_type['id'], list(extra_spec)[0],
            extra_spec)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('49d5472c-a53d-4eab-a4d3-450c4db1c545')
    def test_create_nonexistent_type_id(self):
        """Test creating volume type extra specs for non existent volume type

        Creating volume type extra specs for non existent volume type should
        fail.
        """
        extra_specs = {"spec2": "val1"}
        self.assertRaises(
            lib_exc.NotFound,
            self.admin_volume_types_client.create_volume_type_extra_specs,
            data_utils.rand_uuid(), extra_specs)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('c821bdc8-43a4-4bf4-86c8-82f3858d5f7d')
    def test_create_none_body(self):
        """Test creating volume type extra spec with none POST body

        Creating volume type extra spec with none POST body should fail.
        """
        self.assertRaises(
            lib_exc.BadRequest,
            self.admin_volume_types_client.create_volume_type_extra_specs,
            self.volume_type['id'], None)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('bc772c71-1ed4-4716-b945-8b5ed0f15e87')
    def test_create_invalid_body(self):
        """Test creating volume type extra spec with invalid POST body

        Creating volume type extra spec with invalid POST body should fail.
        """
        self.assertRaises(
            lib_exc.BadRequest,
            self.admin_volume_types_client.create_volume_type_extra_specs,
            self.volume_type['id'], extra_specs=['invalid'])

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('031cda8b-7d23-4246-8bf6-bbe73fd67074')
    def test_delete_nonexistent_volume_type_id(self):
        """Test deleting volume type extra spec for non existent volume type

        Deleting volume type extra spec for non existent volume type should
        fail.
        """
        self.assertRaises(
            lib_exc.NotFound,
            self.admin_volume_types_client.delete_volume_type_extra_specs,
            data_utils.rand_uuid(), "spec1")

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('dee5cf0c-cdd6-4353-b70c-e847050d71fb')
    def test_list_nonexistent_volume_type_id(self):
        """Test listing volume type extra spec for non existent volume type

        Listing volume type extra spec for non existent volume type should
        fail.
        """
        self.assertRaises(
            lib_exc.NotFound,
            self.admin_volume_types_client.list_volume_types_extra_specs,
            data_utils.rand_uuid())

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('9f402cbd-1838-4eb4-9554-126a6b1908c9')
    def test_get_nonexistent_volume_type_id(self):
        """Test getting volume type extra spec for non existent volume type

        Getting volume type extra spec for non existent volume type should
        fail.
        """
        self.assertRaises(
            lib_exc.NotFound,
            self.admin_volume_types_client.show_volume_type_extra_specs,
            data_utils.rand_uuid(), "spec1")

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('c881797d-12ff-4f1a-b09d-9f6212159753')
    def test_get_nonexistent_extra_spec_name(self):
        """Test getting volume type extra spec for non existent spec name

        Getting volume type extra spec for non existent extra spec name should
        fail.
        """
        self.assertRaises(
            lib_exc.NotFound,
            self.admin_volume_types_client.show_volume_type_extra_specs,
            self.volume_type['id'], "nonexistent_extra_spec_name")



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.compute import base
from tempest.common import tempest_fixtures as fixtures
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc


class AbsoluteLimitsNegativeTestJSON(base.BaseV2ComputeTest):
    """Negative tests of nova absolute limits"""

    def setUp(self):
        # NOTE(mriedem): Avoid conflicts with os-quota-class-sets tests.
        self.useFixture(fixtures.LockFixture('compute_quotas'))
        super(AbsoluteLimitsNegativeTestJSON, self).setUp()

    @classmethod
    def setup_clients(cls):
        super(AbsoluteLimitsNegativeTestJSON, cls).setup_clients()
        cls.client = cls.limits_client

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('215cd465-d8ae-49c9-bf33-9c911913a5c8')
    def test_max_metadata_exceed_limit(self):
        """Test creating server with metadata over limit should fail

        We should not create server with metadata over maxServerMeta limit
        """
        # Get max limit value
        limits = self.client.show_limits()['limits']
        max_meta = limits['absolute']['maxServerMeta']

        # No point in running this test if there is no limit.
        if max_meta == -1:
            raise self.skipException('no limit for maxServerMeta')

        # Create server should fail, since we are passing > metadata Limit!
        max_meta_data = max_meta + 1

        meta_data = {}
        for xx in range(max_meta_data):
            meta_data[str(xx)] = str(xx)

        # A 403 Forbidden or 413 Overlimit (old behaviour) exception
        # will be raised when out of quota
        self.assertRaises((lib_exc.Forbidden, lib_exc.OverLimit),
                          self.create_test_server, metadata=meta_data)
